return {
 ["comment"]="% generated by mtxrun --script pattern --convert",
 ["exceptions"]={
  ["n"]=0,
 },
 ["metadata"]={
  ["mnemonic"]="sl",
  ["source"]="hyph-sl",
  ["texcomment"]="% no comment",
 },
 ["patterns"]={
  ["characters"]="abcdefghijklmnopqrstuvwxyzčšž",
  ["data"]=".av5r .di6spo .ek3s .ek5v .is1 .iz1 .obi4d .ob5it .od1 .po4d5n .po4v5s .pre6d7n .se4k5s .si4s .st4 .voz5l .voz5n .zliz6 a1a a1b ab5ba ab6rod a1c ac5ci a1č a1d ad2l a6dobl ad6rl. ad6rla ad6rob ad5ur a1e1 a1f af5ga af1t a1g a1h a4hm ah5mi ah5mo a1i ai2n1 a1j a4j5ek a4jf aj5fi aj5fo aj5ha aj5he aj5im aj6imo aj3os aj6stb a5ju. aj3uč aj3ug aj5žn a1k ak4s a4kst a1l a1m a4mz a1n an6dga an6dhi a4nm an5mi an5zi a1o ao2b1 a1p a4ph a1ra ar6dwa a1re a1ri a1ro a1ru ar5xa ar5xo ar5xu a1s a4sš as5šč a1š a1t a4tf at4i a1u1 a4uf a2uk a4ul a1v av5ši a4vž av5ža ay5to a1ze az5fo a4zig az3la az3le az4lil az4lit az4liv a4zob a4z3oč az5ora az5oro a4zra az4red az5vp a1ž až5mi ba6bba ban3č4 ba4u 2b1c 2b1č 2b1d be1 be4v b1h bi1 b1ja b4ja. b5jel b3jem b5jet 2b1k b3lep b5leta b5lil b5lit b5liv b1m 4bmi 2b1n bo1 bo6chm b5ord bo5vp b3rab b5ras b3raš b3rez bre4zg bre4zi bre4zr b5reže b3rob br6žda 2b1s 2b1š 2b1t bu5ki bu5ku bu5kv bu5ry 2b1v b1z b1ž 2cc 2ch. ch5ma 2ck c1ka ck1o2 c5ko. ckov3 ck1s ck5we 2c1n 2c1t 2č1b 2č1g či1 1čj 2č1k 1čl 4č3let č5mes 2č1n 4čop 2č1p 2č1s 4čup 2d1b 2d1c 2d1č 2d1d dd6voj d2e 6d5elem de4min de4mn de4z3i 2d1g 2d1h di5ck 4dind d4i5no dis1 di4skr di6spr 2d1j 2d1k 5dlet d2li d5lit d5liv d1lo 2d3m 4d3nac 4d5nač 4d5nap 4d3nar 4dnas 4d5neb d5niv 4d5niz 4d5njač 4d3nož d2o 4dobč 4d5obd 2d3o2f do5rd do5vč do5v4z 2d1p d5raz d3rep dre6pn d4rev 2d1s 2d1š 2d1t dteks6 d4ur du5ro du5um 2d1v 4d3vi 2d1z2 e1a e1b eb4j eb6liz e1c e1č e4čd eč5de eč5di eč5do eč3le eč5op e4čt eč5ti eč5to eč5tr eč5up e2č1v eč6vrs e1d e4df ed5ig ed2l ed5ob ed6obe ed6obr e4dobs e4d3oč ed5vč ed5zb e1e e4ep e1f e4ff ef5fe ef5ta e1g e1h e1i ei6pzi ei2z eiz5e e1j e1k ek6mal ek6tre e1l e1m e1n e1o1 eob4j eob4r eo4dl eo4z5n e1p ep5nik e1ra era6z5l era5z4r era5z4v e1re e4rf e1ri e1ro e4rr e1ru e1s es5da e5sta e5sti. e5stih e5stil e1š e4šp eš5po e1t 4eth e4tinš e1u1 e1v eve6t5l ev5ha ev6pre ev6ste ev5stv 2ew ew6ind ew5le e4wt ew5to e4yw e1z ez5dj e3z4dr ez2g ez5gl e5zij ez6ijo ez5imn e5zis ez6ist ez5iz ez4l ez6lom ez6man ez4mo e4zob e4z5or ez4re e4zt e4z5u4m5 e4zž e1ž 1fa fe1 fe6ljt ff5ma fi6zlj 2f1n fo6uri fre4u 2f1s 2ft ft5ve fu1 2g1d ge6ige gel5č4 ge6njč gi6tpr go1 go5vz 2g1t gu1 ha4u 2h1č he4i 2h1k 4hl. h4lo 2h1n h5ren 2h1š 2h1t 1hu hu6ffm i1a i1b i1c i4cs i1ča i1če i1či ič5ra i1ču ič5vr i1d 4idor i1e1 i1f i1g 4igh i1h i1i ii2n1 i1j i1k i4kč ik5ča i1l il5č4k 4ile 4ilo i1m i4mh im5hi i1n 1ind 2ine 3i4n3os 1inp 3inse 1inš 4inšk 3intr i1o1 i1p i1r 4ire i1s is4a is6ert isis4 i4skv 2iss i1š i1t it5pr i1u i1v iv5jo i1x i1z iz1l iz4la izli4z iz5me iz5mo iz6ode iz5po i2zr iz1u iz6ure i1ž j5akt 2j1b 2j1c 2j1č 2j1d je4ks4 2j1g 2jh j1hi 4jime 4j5int 2j1k 2j1l 2j1m 2j1n 4job 2j1od jod4l 2jos 4jož 2j1p 2j1r jra1 jraz4 2j1s jsis6t 2j1š 2j1t ju1 2juč ju5dm 2jus ju2ž1 2j1v 2j1z jz6ves 2k1c 2k1d ke5ti ki1 2k1m 1kn ko1 kok4 ko5kd ko6vše koz6lo 1kre 2ks. k5sat ks1c ks1p ks4po ks1t 4kst. ks6taz ks5te 2k1t 3ktr 4ktra ku5ro k5vip la4ir la6vz. 2l1b 2l1c 2l1č 2l1d le1 le4e le6ipz le5me 2l1f 2l1g lg5ča 2l1h l2i1 li6dž. 1liz 4l5izd 2lj. 4ljc 2ljč 2ljk 2ljn 2ljs 2ljš lju5d6j 2l1k 2l1l 2l1m 2l1n lo1 1loč 2l1p 2l1s 2l1š 2l1t lu5ki lu5ku 2l1v 2l1z 2l1ž 2m1b 2m1c 2m1č 2m1d me4d5n me6dos me4dr 2m1f 4mind 4minp 4minš mi6th. 2m1k 2m1m m5niv mo6št. mo6vš. 2m1p 2m1s 2m1š 2m1t m5urn 2m1v my5hi 2m1ž na1 5načel na4d5nj nad5r na6dra na4dre na6dur 1naj na6jak na4j5en naj3o na6joč na4j3u 1nas na4v3z navze6 1naz naz6or 2n1b 2n1c 2nč n1ča n1če n1či n1ču 2n3d2 nd5ga nd5hi n4dm ne1 ne3d2 1neh ne3zm nez4v 2n1f 2n1g n4gh ng5ha n4gv ng5vi 2n1h 2nj. 2njc nje4v5s 2njk 2njs 2njš 4njv 2n1k 2n1l 2n1n no5rd n4ost 2n1p 2n1s nsis4 2n1š 2n1t nteks4 n4tg nt5ga nt5ge n4tv nt5vi nu1 2n1v ny5qu 2n1z nz4i 2n1ž o1a o4as o1b ob5gl ob5ide ob5jo 5obla 5obro o4bz o1c oc5ke oc5ki o4cr o1č o1d od5dv od5nal o6drep od5zd o2d1ž o1e oele4 o1f o1g 4ogl o1h o1i oiz2 o1j o1k o4kb ok5ba ok5be o4kt o1l o6l5avt ol6gča o4lr ol5re o1m o1n o1o ood4l o2ol o4om o1p o4pm op5me 4opy o1ra or4deč o1re o1ri o1ro o1ru o1s 5oseb ose4m5 o1š o1t o1u ou5ki ou5ku o1v ov5sem ov5šk o2v1z o5vza ov3zd o1y o1z oz4b ozd5j oz4g oz5lo oz6lož oz2n oz5nic oz5niš oz2o oz2r oz2v o1ž ož5mi 2p1c 2p3č2 pč5ka pe1 1peč pe4kt pet3l pe4tle pe4v5s pev5t4 4phs ph5so pi5zo 2p1k 4ploz po1 po6dfa po4d3l po4dna po4d5oč po6lob po6std prez4 2p1s 2p1š 2p1t pz6ig. qu2 3raču 2rae ra6jžn rav5z ra6vza ra4z5id 3razl ra4z5or 2r1b 2r1c 2r1č 2r1d re1 3real re6cht re5čv 5redč re6dig re6dnju re6iba re5jo re5km re6sda rev6sk re6znač re6zus re6zve r1f 2r1g 2r1h ri1 r4in ri5n4o riz4g riz4l riz4n 2r1j 2r1k 2r1l 2r1m 2r1n ro1 rob6id 3rodi ro5zo 2r1p r1r 2r1s 2r1š 2r1t r4th rt5ha ru5kl 2r1v r3v2j rv5jo ry5an 2r1z rz2l r1ž rž5da 2s1b 1sc 4sc. s2ci se4k5sa sek5si se5ma se5vp 2s1f si1 s4id si6gn. sis1 2s1j 2sk. s2kn 4skre s4lav s4on soni5 sonič4 1sp s4plod spod4l 2s1s 2st. 3ste s4ten 4stf s4tič 5stim s4tir 2stk 2stm 1str s4tra. su1 su4bo sve5t š2č 2šč. 2ščk 2ščn še2s 2š1j ta5wi taz4 2t1b 2t1c tch5o 2t1d tek6st 5tema te5xa t1f 4tind 4tinos 4tinp 4tinse 4t3int 2t1k 6tletno 2t1m 4tnaj to6vž. trt5u tr6tur 2t1s 2t1t tu1 4tz. 2u1a u1b ub4j u4bp ub5po u1c u1č u1d ud6mi. u1e u1f u1g u1h u1i u1j u1ka u1ke u1ko u1l u1m u1n u1p up6čka u1ra u1re 4urg u1ri u1s 1usp u1š uše3s u1t u4th uth5o u1v ux5em u1z u1ž 2v1b 2v1c 2vč v1ča v1če v4čer v1či 2v1d ve4čl ve4čm ve4i ve4tin vetle6t v1f v1g vi5dv vid6va 1viv vi6žg. 2v1j 4vjo 2v1k 2v1m 2v1n vo5rd voz5le 2v1p 3v2pa v4pij v4pil v5skn v5šek 4všk 2v1t vt4k vz2 v2za 3v2zg 2v3zk 2vzo v3zp v2zu 1wa wo2 x1f 1ye 2y1f y1j y1l y1w 1z2a z6ane. za5uk za3vp za1z2 za5zd 2z1b 3zbir z1c 2z1č 2z1d2 zd5ju z3dv z1g z4gni z5got 2z1h 1zi z1ig 2z1is 4z5išč 2z1j 2z1k z3ku z5las z1li 3zlil 5zlit 5zliv zliz5 1zlj 3zlog z5lom 3zlož z1lu 2z1m 1zn 1zo z1ob 2z1od z1og z2ol z4om 2z1p 1z1r 4zredč 4zreš 4zrez 4zrež 4zri 4zru 2z1s z1š z1t 1zu z4uj 2z1up 2z1uz z1v2 z4ven z3vn 3z4voj z4vok 2z1z2 z1ž 2ž1b 2ž1c 2ž1č 2ž1j 2ž1k 4žmi .č8 .š8 .ž8 8ž. 8š. 8č.",
  ["lefthyphenmin"]=1,
  ["length"]=6052,
  ["n"]=1068,
  ["righthyphenmin"]=1,
 },
 ["version"]="1.001",
}