return {
 ["comment"]="% generated by mtxrun --script pattern --convert",
 ["exceptions"]={
  ["characters"]="acefhijklnoprtyíúčř",
  ["data"]="koe-fi-ci-ent koe-fi-ci-en-ty pro-jek-ční úhlo-příč-ka úhlo-příč-ky",
  ["length"]=77,
  ["n"]=5,
 },
 ["metadata"]={
  ["mnemonic"]="cs",
  ["source"]="hyph-cs",
  ["texcomment"]="% no comment",
 },
 ["patterns"]={
  ["characters"]="abcdefghijklmnopqrstuvwxyzáéíóúýčďěňřšťůž",
  ["data"]=".a2 .a4da .a4de .a4di .a4do .a4dé .a4kl .a4ko .a4kr .a4ku .ale3x .a4ra .a4re .a4ri .a4ro .a4ry .a4rá .a4sa .a4se .a4so .as3t3 .a4sy .a4ta .a4te .at3l .a4to .a4tr .a4ty .a4ve .b2 .c2 .ch2 .cyk3 .d2 .dez3 .d4na .dne4 .dneš4k .d4ny .dos4 .d4ve .d4vě .d4ví .e2 .e4ch .e4ko .es3k .es3t .e4ve .f4ri .g2 .h2 .h4le .h4ne .i2 .i4na .i4ni .i4no .is3l .j2 .j4ak .je4dl .j4se .j4zd .jád4 .k2 .k4li .k4ly .kří3d .l2 .le4gr .li3kv .m2 .mi3st4 .moud3 .na3č4 .ne3c .neč4 .ne3š .ni2t .no4s3t .n4vp .ná1 .nář4k .o2 .o4bé .ode3 .od3l .od3rá .o4ka .o4ko .o4na .o4ne .o4ni .o4no .o4nu .o4ny .o4ně .o4ní .o4pe .o4po .o4se .o4sl .os4to .os3t3r .os4tě .ot3rá .ot3v .o4tí .o4tř .ově4t .o4za .oz3do .o4zi .o4zo .o4zu .o4šk .o4šl .o4ži .p2 .pa4re .pa3tř .polk4l .po3č4 .p4ro .p4rý .p4se .pu3b .r2 .rej4 .re3s .ro4k .roze3 .roz3r .ru4dl .s2 .s4ch .s4ci .sem4 .se3pn .s4ke .sk4l .s4ká .s4le .s4na .s4ny .s4pe .s4po .st2 .s4tá .s4ži .t2 .u2 .u4ba .u4be .u4bi .u4bo .u4de .u4di .u4do .u4du .u4dí .uh4n .uj4m .u4ko .u4ku .ul4h .u4ma .u4me .u4mi .u4mu .u4ne .u4ni .u4pa .u4pe .u4pi .up4n .u4po .u4pu .u4pá .u4pě .u4pí .u4ra .u4ro .u4rá .us2 .u4so .u4st .u4sy .u4sí .ut2 .u4vi .u4ze .u4če .u4či .u4čí .u4še .u4ši .u4šk .uš4t .u4ší .u4ži .už4n .u4žo .u4ží .v2 .va4dl .v4po .vy3 .v4zá .vý1 .v4ži .y4or .y4ve .z2 .za3 .zao3s .zar2 .zač2 .zd2 .z4di .z4dr .z4ky .z4mn .z4no .z4nu .z4ně .z4ní .z4pe .z4po .z4tř .z4ve .z4vi .č2 .č4te .é2 .í2 .ó2 .š2 .še3t .š4ka .š4ke .š4ky .š4ťo .š4ťá .ú2 .ú4dů .ž2 a1 2a. aa3t2 ab3lon ab4lý ab3ri ab4sb ab2st ac4ci a2d a3da a3de a3di ad2la a4dli a4dlá a4dlé ad4me ad4mu a3do ado4s a3d3ra ad3ri a3drž a3du a4duž 3a3dva ad3vo a3dy a3dá a3dé a3dě a3dí ad4úz ad4úř a3dů a3dý ae4vi afi2a a2g a3ga ag4fa a3go ag3ro a3gu a3gá ah4li ah3v a2i a3in ai4re a3iv a2jd a2jm aj4me aj2o a2k a3ke a3ki a3kl ak4ni a3ko a3kr a3ku a3ky a3ká a3ké a3kó a3ků a3ký al4fb al4kl al4tz al3ží am4bd am4kl am4nu amo3s am4ži a4nae a4name an4dt ane4sk aneu4 an4sc an4sg an4sl an4sm an2sp an4sv an4tč an4žh ao4ed ao4hm ao4stř ao4tč ap4r. a4pso ap3t a4př. a2r a3ra ar4dw a3re a4rer ar4gl a3ri ar4kh a3ro a4rox ar3st a3ru ar2va a3ry a3rá a3ró ar3š2 ar4šr a3rů arůs3 a3rý a2s a3sa a3se a3sh a3sin as3ná a3so as3pi as4tat a4stk as4tm a4stru. as3tv a3su a3sv a3sy a3sá a3sé a3sí a3sů a2t a3ta at4ch a3te a3ti a4tio at4kl at3lo a3to a3tr at3re at3ron at3rov a4tru at4rá at4th a3tu a3tv a3ty a3tá a3té a3tě a3tí a3tó at1ř a4tří. a3tů a3tý a2u au4gs a3uj auj4m aus3t a3uč 2av av3d av4d. av3lo a4vlu a4vlí av3t av4ti 2ay ay4on az3k az3la az4lé az3ni a3zp a2č a3ča a3če a3či a3čl ač4má a3čo a3ču a3čá a3čí a3čů a2ň a3ňo a3ňu aře4k a3ří a4špl a4špy a2ť aú3t 2b. 3ba. ba4br ba4chr ba3ka ba4se 2b1c b1d be4ef be4et bej4m be3p beu4r be2z3 beze3 b1h 1bi bi2b3 bis3 bist4 bi4tr b1j 2bk 3bl. bl4bl b2lem b2les 3blk b4lán b2lém b1m 2bn 1bo bo4et bo4jm bo4ok bo4tr bou3s bo4šk b2ral b2ran 2bri b4rodit b4rou broz4 b2ru b3ru. b3rub b2rán 2b1s2 bs3tr 2b1t btáh4 bu2c bu4en 3by. bys3 by4sm by4tč by4zn b2z 1bá 2b1č bé4rc 1bě. bě3ta 1bí 3bín bí4rc 2bň b3řa b3ře. bře4s b1ří 2bš2 2c. 1ca cad4l ca4es 2cc 1ce cech4 ced4l celo3 ce4ns ce4ov ce4ps cer4v ce2u 2ch. 1cha 4chalg 3che 4che. 2chl ch4ly ch4mb 2ch3n 2cht 4chte 1chu ch4u. 1chy 1chá 2chř 1ci cien4c cik4l 2ck2 c4ket ckte4rý 2cl c3la c3lé 2cn 1co co4at co4mm co4žp c2p 2ct c2ti ctis4 ct4la ct2n c3tv c2tě cuk1 1c2v cy2 1cá 1cí cí4pl 2cň 1ců 2d. 1da da3d da4jš da4kl da4tr d1b d2ba 4dbat. d2bá 2d1c dch4l 3dch4n d1d dd4ha 1de de4bre de3hn de3jd dej4mo de3kl de3kv de2na de2oz de3sl de4sm de4so de2sp des4t de3str de1x de4xt de2z de3zn dez3o de3čt de4žp 2d1h 1di di4gg 4dind dis3k di4so d1j dj4us 2dk d3kv 3dl. d1la d4lab d4lak d3li 1dln d2lou d3lou. d2lu d3luč d4láž d1lé 2d1lí d2lů d1m 1dmd dmýš4 2dn 1do 4dobl 4doboj dob4rat do3by do3bě do3bý do1d 4do4dd 4do4dj dod4n do3h doj4m 4dokn 4doly do3mn domoh4 do3p do4pc dop4n dor2v do1s dos4p dos4tiv do3t do3uk do3uč do3z2 doz4n do3č 4do4čn doč4t do4žp 4dran d4rap d1re d4ren 3drobn d3ros d3rou d3roš dr4sc d3ruš d3ré d3rý d4rýv 2d1s2 ds4ků ds4po d1t d3tl d3tř 1du dum3ř du3na du3p du4pn 2dur du3si du4í. d2v d4vac d3ve d3vl d3vr d3vy d3vá d3vě d3ví 1dy dy4su d3zb d3zd d3zn 1dá 2d1č 1dé 1dě 3děj 1dí 2dň d1řa dře4k d4řep dře4pn d4řev d1ří d2řít 2dš2 d3šk d3št 1dů 3dů. dů3s 1dý d2ž2 2e. e1a ea3dr e2ar e1b eb4er ebez2 eb4li e2bř e4ch. e3chl. e4chm e3cho e2chr e3chv e4chť ed4be ed4kv ed1l ed2ma e3dmn ed3v ed4ří e1e ee4th ee3xi eg4gi e1ha e1he ehno4 eh4ně e1ho e1hr e1hu e1hy e1há e1hý e1i eilus3 ej3ag e3jas e1je e3jed ej3ele e3jez ej3in e3jis ej1m ej3mo e3jmu ej1o ej1u eju3st ej3v e2k e3ka e3ke e4kly e3ko e3kr e3ku e3ky e3ká e3ké e3kó e3kř e3ků e1la e4lau el4dv e1le e1lo e1lu e1ly el4ze e1lá e1lé e1lí e1ml e4mlí emo3k e1mr e1my e3má e1mě e1mí e3mř e3mů e1mý em3že en4dv enitos4 en4sc en4si ent3r e1o eo3by eoch3r eod3l eo4du e4ole eo1s eo2st eo4tř eo3z eo4zb eo4zd eoše3 epa3t e2pl e4pni ep2no e4pný epoč3t epro4zř ep4tl ep4tm ep4tn e4ptu epy3 2er e1ra er4a. e1re e1ri e1ro er3s er4s. er4sn e1ru e1ry e1rá e1ré e1rů e1rý e1s e4sag e2sce e4sin esi4s e2sk es4k. e4s4kn es3ku. es3ky es3ké e2sl e4s3li e4sly es2m e4sp. es4pe e2st e4st. e4ste es3tiž es4tol e4strou es3tán e1t e4tki e4tkr e4tli e4tly et3ri et3ro et3rů et1ř et4ún e1u eu3b eu3ct eu3d eu3k eu3m eu4m. eu3n eu3p eu3r eu4r. e4ura eu4ras eu4rg eu3s2 eu3t e4u4t. eu4tra eu4ts eu3v eu3z eu3ž e3vd eve4š e3v2k e4vsk evy3 evyjad4 evypá4t evy4čk evě4tr ex4ta e3xu ey4or ey4ov ezaos3 ez4ap ez4bo ez3de ez3dov ez3du ez4dě e3ze ez4ed2 ez4ej ez4el ez4er ez4es ez4ez ez4eš ezis4 ez4it ez4le ez4ná ez4ně ez4py ez2t ez4ác ez4áh ez4če e3zí e3zř ez4ře e1á eč4kat e1čt eč4te e4čti e4čtí e2ň e3ňo e3ňu e3ňá e3ón e1ř eře4k eř4ku e3ří e2š e3še e3ši e4ška e3šl eš4lá e3šo eš4to eštíh4 e3ší eú1 eúmy4 eú3n eú3p eú3t eú3č ežíš4 1f 2f. fe4in fene4 fe4ue fi4em fi4fl f2l f3lí fló4r fm4no 2fn 2fr f4ran f4ras 3frek f1ri 2fs fs4te 2ft fu4ch 2fé f2ú 1g 2g. ga4uč ge2s ghou4 3gic 3gin gi4ím g4lom 2g1m 2gn g4noi g4nos go1 go4hm 3graf gu4el gu4it gu3m gu4m. gus4t gu3v 2h. ha4ag ha4ar ha4bl ha4br ha3dl ha4dla ha4ke has3t hatos4 ha4yd h2b h2c 2hd he4br he4id hej4s he2s he2u he3x hi4an hi3er hi4gh hi4re 2hk 4hla. h4led h3len 2hli 4h3lo. h3lob h3lop h3lov h3luj 2h1ly 4hlá. h4lás h3lí. 4hlík 2hlý h2m 2h2n h3ne h4ned h3niv h4noj 3hněd 3hodin ho3str hos4tě 4hove 4hovna 4hovny 4hovná 4hovně h2r hra4p 2h1t h4tin h2tě h4tít hu4ch hu3mo hu4tň 2h2v hyd1 hy4do hy4ps hys3 hy2t3r hy4zd h1č 2hň hř2 hř4by hý4bl h2ž 2i. i1a ia3d ia3g2 i4al. ias4t ia4tr i1b ib2l i2b1r i1ch i4chž i1d id4ge id2l id4lo. i4dlý i1em i1en i1et if1r ig4ne i1h i2hl i3hl. i4hli ih3n ih4na i3im i1j ijed4 ij4me ij4mi i2kl ik3le ik3lo. ik3m ik4ry i4kve ik4úř i1l il4ba ilič4n i4lnu ilu3 i1m i4mla i4mly i4mun i2n i3na ina3d in4cm in4dl i3ne 3infe in4gh in4gp in4gs in4gt i3ni i3no i3nu i3ny i3ná i3né i3ně i3ní in4šp i3nů i3ný i1o io4sk i2ps i1r iro4s i1sa is3c is4ch is4k. is3ka is3ke is3ko. is3kr is3ku is3kv is3ky i3slav is3lo is3lé is3pl is3po is1t is4tal is4tat is4th ist3v is3tí i1sy i3sá i1t it1r it4rh it4rp it4se it4su i2tv i1um iv3d i1x ix4td i3zp iz1r i1á i1čl ič3t ič4tl ič4to i2ď i1é ié4re. i1íc i1ím i1ó i1ř iř4kl iř4če i2š i3še i3ši iš3k iš4kr iš4kv i3šo iš4to i3šu i3šá i3ší i2ž i3ža i3že i3ži i3žo i3žu i3žá 2j. ja2b2 jac4k ja4cq ja3d ja3g j3akt j1b2 jbyst3 2j1c j2d j3dob j3dok j3dos j3dr j3dá jd4ří j3dů jech4 j3ef j3ex jez3dí jg4ra 2j1h 1ji ji4ch jih3l ji4mž j4ina jis3k jit4ro ji2zv j1j 2jk j3kv 2j1l j2m j3ma j3mi jmou3d 2jmí 2jn jne3 j1ob j1od jod2ř j1oh j1op j4ora j1os jo3sv j2ov j3ovl j1o3z2 2jp jpor4 jpo4zv jpříz4 2j1r 2j1s2 j4sem j4si. j4sk. js4ko js4ká j4s4ků j4s4me j3sn j4sou. j4souc js4po j4s4te 2j1t j3tl ju4an ju3na ju3p j1us ju3sp ju3t ju4t. ju3v ju4xt ju3z j1už ju3ži 2jv2 j3vd j3vn 2jz j3zb j3zd j3zk j3zn j3zp jád2r 2j1č 2jď 1jí j3št jš4ti j3šť 2jú1 jú3n jú3č jú3ž 2jž 1k 2k. ka4bl ka4ch ka3dl 3kaj ka3ka 3kami 3kaně ka2p3l ka2p3r ka2ps ka4pv ka2př kas3t kast3r 3kat ka4uč 3kav 3kač 3kař kaš3l ka4šp 2k1c k2d k2e ke4bl ke3jo ke4pr ke4ps 3ket 2kf 2kk k2l 3kl. 4k3la. k3lej 4k3li. k4lib k3lic 4klička 4klo. k3los 2k3ly k3lá. k3lé k3ló k3lý 2k2m k3mě 2kn kna4s ko3by 3kof ko4jm ko2př ko4sk ko2t3v kous3k 3kov ko3zá 4kroa k3rob k3rof kr2s kr4ú. 2ks 2k1t kt2r kuch4 ku4fř ku4hr 3kuj ku3se ku3si ku3su ku4th ku3v 2k2v k4vrň 3kyn ky2pr kyp3ř ky4zn 3kác ká4pl 3kár 3kář 2kč k2ň k2ř2 k3řej kš4ti 3ků. 2l. 1la. la4br lab4s la3ka la4nq la4ps 4la3si la4vš la4y. la2zm 2l1b 2l1c 2l1d ld4ne le4ad le4au lech3t leh3n le2i 1lej le3jo 4lejšk 1lel 4lench lepa3d lepo4s le4pr le4ps le4sc le4sm le4sv let4li let3m le2tr le4tč le4uk le4vh le4vk le3xi lez3n 2lf 2lg 2lh 3lhan 1li li4az li4bl li4bv li4dm lind4 3lio li4tň li4vr 2liž 2lj 2lk l4kat l2kl lk4nu 2ll 2l1m 2ln l4nul lo3br lo4id lo4is 1los lo3sp lo3stř lo3sv lo2tr lo4tř lo4u. lo3z loz4d lo4šk 2lp l2pě 2l1s2 l4sla ls3n lst4n l4stí 2l1t lt4ra lt4ru lt4ry lu4id lu4j. lu4k. lu4lk lu4m. lu4mn lu3pr lu3va lu3vl lu3vy lu3ví 2lv 2lz 1lá. lá4jš lá4vš 2l1č 1lé. 1lík lí4pl lí4zn 1líř 2lň 2lš2 l3št l4štý 1lů 1lý lý2t 2l2ž 2m. 1ma maj4s ma4kl ma4kr 4mald mas3k mat3r ma4tra ma4vš maz3l 2m1b 2m1c 2m1d2 m2dl 1me 3me. me4go me4is met3re me3x mezi3s 2mf mh4le 1mi mid3l mik3r mi4xt 2mk2 3m2kl mk4la mk4li m2l 4mla. 2mle ml3h ml4h. 2mli ml4sc ml4sk 4mlu. 2mn m3na mna4s m4noh m3nos m4noz 3množ m3ná m3né m4néz m3něj m3ný 1mo mod3r mo2hl mo2k mo2s mo4s. mot3ř 4mout moza4 mo3zř moú3 2mp m4plo mpo4s m2ps mp4se mp2t mr2s 2m1s2 m4stl 2m1t 1mu mu4fl mu3n mu4n. mu4nd mu4nn mu4ns mu4nš 2muš 2mv mys3lo my4šk 2mz 3má. málo3 má2s 2mč m2če mí1c mí4rň 2m2š mš4či mš3ť mš4ťan. 3mů. 3mý. m2ž 1n 2n. 3na. na3ch na4do na4em na3h na4h. na3jd na3ka nam4ne na3p2 na3s2 na4s. nat2 na3tl na3tř na3z naz4k na4zš na4č. na3š naž4n 2nb 2n1c n4chc 2n1d nd4hi ndo4t nd2re nd4ri nd4ří ne1d ne4gl ne1h ne3h4n ne2j nej3t nej3u ne3kl ne4kro ne3kv ne4m. ne3p ne3s2 ne4s. nes4le ne4ss 4nesti ne3tl net4r ne3ud ne3v2 ne4v. ne3z nez4n ne3šk ne3šť 2nf n3fr 2ng ng1l ng4la ng4le ng4lí n4gro ng4vi nik4t ni4mr ni4mž 3nio 3nisk 2nitř n1j 2nk 2n1l 2nn no3b2 no4bs no3hn no4hs no4ir no4mž no4sky no3sm no3str not4r no3z no4zd no4šk 2nož 2n1s2 n2sa ns3ak ns4ko n4soc ns3po nst4ra 2n1t nte4r3a nt4lem nt4r. nt3ru nt3rá 2nub nu4gg 3ny. 2nz 3nák ná3s2 ná4s. 2n1č 2nď 2nív 2níž 2nó 2nš2 n3št nš4ťo nů2 2nž 2o. o1a oang4 o1ba o1be obe3j obe3s obe3z ob1l ob1r ob4rň o1bu obys4 ob3z o3bé ob3řez o1c o4chl o2chr oc4ke oc4ko o4ct. oct3n ocy3 oc4ún od3b odej4m ode3p ode3s od1l o4doc odos4 odo4tk od3ra od4ran od3rů o3drž od3v od1ř o1e2 oe3g oe3ti o2fl ofrek4 og2 o3gn o1h oh4ne o1i oi4ce o4int o1j o4jar oje4dl o4jmi o4jmov o4jmu o4jmů oj2o o4juz 2oka ok2te o1l ol4gl ol4to o1m om4kl om2n o2n o3na ona4s o3ne o3ni o3no ont4ra o3nu o3ny o3ná onář4ka o3ně o3ní o3nů o3ný o1o oo4hř oote2 opoč3t opro4s o2ps o4ptu opá4t o4př. opřej4 opře4jm o1ra o4rae or4dm o1re o1ri o1ro or3st o1ru or4vá o1ry o1rá o3ré o1rů orůs3 o3rý o1sa o4sai ose4s osi4d o1sk o4s3ke o4sku osk3v o4ská o4ský o1sl os4la os4li os4lý os3mo os4mu o4st. o4stg o4stm os4tor os3trů o4sté o4stš o4stý o1sy o1t ot4kl o4tlý oto3s ot3ro ot3ví o3tí o3tř ot3ři o2u ou3bě ou3dě ou4fl ou4il ou4is ou4k. ou3ka o4ukl ou3kr ou3ká ou3m oup3n oupo4 ou4s. ou3sa ou3se ou4sk ou3sm ou4tv ou3v ou4vl ou4vn ouz3d o4učk ou3ži ovi4dla o4vsk ovy2p o2všt o1x o2z o3za oz1b oz4d. oz3dá oz3dě oz3dí o3ze oze3d2 ozer4 oz1h o3zi oz3j oz3k oz4ko oz1l oz3m o4zn. o3zo oz3p oz4py oz4pě oz4pí oz3ro oz3ru oz3rů oz3t o3zu o4zut oz3vr oz3vá o3zí o3zů ozů4s o1č oč2k oč4ka o2ň o3ňa o3ňo o1ř oři2s o3šk o4šku o4šky o3šl oš4lá oš4mo oš4ti oš4ťu o3žl ož4mo 1p 2p. pa4ed pa4es pa4kl pa3si pa4t. pat4ri 2p1c pe4al pede4 pe4ig pe4np peri3 pes3t3 pe4tra 3peč pi4kr pi4pl 2pk p2kl p2l 3pl. 4p3la. pl3h pl4h. 4p3li. 4plo. 2pn p2nu po1b2 po3c2 3pod podbě4h pod4nes po3dru po3drá po3h poly3 po3m2 po4mp po4ol po3p po4p. po4pm po1s2 pos4p post4r po3t2 po4t. po4tn po3uk po3uč po3už 3po3v po3z2 po4zd poč2 po3čk poč3te po3ří po4šv 2pp 4pra. pra3st pr2c pro1 prob2 pro3p pro3t4 pro3z pr2s 4prán prů3 pse4s 2p1sk p4sut 2pt p4tej p4ter p4tev pt4ri p3tu p4tá. pu4dl pu4tr pyt3l pá1 pá2c pád3l pá4nv pá4sl 2pč pé4rh 2př. pře3h pře3j pře3t4 pře3z pře3č2 při3 přih4 2pš pš4ti 2pť qu2 2r. 1ra. ra4br ra4em ra4es ra4ff ra4hl ra4hm ra4jg ra4jš 2rak ra4nh ra3si rast4r ra4vv ra4wl ra4y. ra4yo ra4ďm 4raži r1b r2bl r1c rca3 r3cha r3cho rc4ki r1d r4dla rdo2s re4ad re4au red4r re4et re3kl re3kvi re4mr re2sb res3l retis4 ret4r re4um r1ha r3hl. rh3n r1ho r3hu r1há ri4bb 1ric ric4ku ri4dg ri4dr ri4fl ri4gh ri4zm 2rk r2kl r1l 2r1m r4mio 2rn rna4vš rn4dr ro4ad ro3by rod2l ro3d4r 3rofy ro3h ro4h. ro4jb ro4kš rom3n romy4s ropát4 ro2sb ro4skv ro4sky ro3sv ro3ti ro3tl ro4tč ro3vd rově4t 3rový roz3d roz3n ro4zo roz3v ro3zá ro4čp rpa3d 2rr rr4ha rr4ho 2r1s r2st r4stu rs3tvě rs3tvý 2r1t r2th r4trá rt4sm rtu3 r2t3v rt4zu 1ru. ru3se ru3si rus3k ru3ži 3rvaní r1x 1ry. rych3 ryd2 rys3ky rys3t ry4zk ry4zn ry4í. ry4šk 2rz rz3d rz3l rád4l rá4dž 1rák rá3ri 1rář r1č 4rčitý. rč3t 3ré. 2ró 2rš rš4ní rů4m. růs3ta rů4v. 3rý. rý4zn 2s. sa4pf sa4pr sas3k s2b2 s2c s3ca s3ce. sch2 sch4l sch4n 3schop s3ci sci4e s3cí s2d 1se se4au se3h se4ig se4il sej4m se4ku 3sel se3lh 3sem ser4va se3s2 ses4k se4ss se4stra se4stru se4stř set2 se3tk se3tř se4ur se3z se3čt 2sf s3fo 3sfé s3fú 1si 3sic 3sif si4fl sig4no 3sik si3ste 3sit s2j s3ju s2k 4skac s4kak 4skam s4kok 2skon skos4 4skot sk4ra sk4ru sk4ry 4skve sk4vo s3kán s3ků 3sl. 4s3la. s4lav s3le. s4led s3lem s3len s3let s4lib s4liči 3sln 4s3lo. s2ly s3ly. s1lí s2ma s4mek s2mo 2sn s2na s3nat s2ne s3ne. sn4tl s2ná s3ná. s4níd 1so sob4l so3br so4sk so4tv sou3h sou3s souz4 so4šk s2p s4pol spro4s 1sr 2ss ss4sr 2st. 4sta. s3taj s2tan st4at 4stec s4tep st4er s4tero s4tich 2stil s4tink 4stit. 4stič st3lo 2stn 4sto. s4tona 4stou. 4str. 4stram s4trik 4strn 4strác 4stupni s2tv st4ve 3ství 4sty. s4tyl 3styš s2tá 4stá. s3tář 4stě. s4těd 3stěh s2těr s2těž s1tí 2stí. s3třej 1su su4ba su4bo suma4 su3ve s2v sy3c sych3r sy4nes sá2d 3sáh sá2kl 2s2č s3či 1sé 1sí 2sň 2sť s3ťo 1sů s2ž 2t. 1ta. ta2bl tac4tvo t2a3d 1taj ta4jf ta4jg 4talt 4tand 3taně t1ao 2tark tast4 ta3str ta4čk 2t1b 2t1c 1te 3te. te4ak te4fl te4in 4teném teob4 tep3l ters4 tes3ta te4tr te4uc te4ur te4ut 2tf 2tg 1ti ti4gr 2tih ti3kl tin4g ti4pl ti3sl tis4tr ti4tr 2titu tiz4r 4tizí tiú3 2tiž 2tk2 t4kal 4t2kan t4kat t2kl tk4la tk4li 4tkně t2ká 2tl 3tl. 4tla. t1le tles3 3tlm t3lo. t4lou tlu3 tlu4s t1ly t1lé 2tm t2ma 2tn t3ní 1to to4as to3b tob4l to3dr to4hm to4ir 2toj tol4s to4ol 4top. 4topt 4topu 2torn 2toup 2tp t3rant t4rea t4ref tre4t 4tric. trip4 t4rit t4rog t3rol tro4sk t4rou 4trouh 4troň. 4trun t4rus 4t4ruž t3ráln 4tráš 2trč t3rům t3rův 2trý 2t1s ts4ko ts2t 2t1t tt4ch tt4ri 1tu. tu4ff 1tuj tu4lk 2tup tu4r. tu3ry tu4s. tu4ť. tu3ži t2v 2tve 2t3vi t4vinn t4viš t4výc 1ty. ty4gř ty2la ty4ře ty4řh ty4řj ty4řo ty4řr ty4řú 3tá. tá4fl t2č t3či 2tčí 1té té2bl 3tém 1tě tě3d4l 2těh 2těnn 2těp 1tíc 4tíc. 4tíce 1tím 2tín 2tír 2tř t4řeb třeh3n t2řel t2řic t3řil tř4ti t1řu t2řá 3třáb tří4s 2tš t3št tš4ti 1tů 1tý. 1tým 1týř 3týš u1 2u. u2at u2b u3ba u3be u3bi u3bo ubs4t u3bu u3bá u3bí. u3bů uc4tí 2u2d u3de u3di u3do u3dru u3du u3dy u3dí ue4fa 2uf u2hl uh3lá uh3no u2in u2jm u2k u3ka. uk4aj uk4al uk4at u3ke uk3la uk3le u3ko u3ku u3ky uk4á. u3ků ul4fa ul1h ul4pí u2m u3ma u3me u3mi um4pl um4ru u3mu u3má 3umř u2n un4dl u3ne u3no u3nu u3ně u3ní u3nů un4žr u2p u3pa u3pe upe2r3 u3pi u3pln u3pu u3py u3pá u3pě u3pí u3pů u2r u3ra u3re u3ri 2u3ro u3ru u3ry. u3rá 1urč u3rů u2s us3ky us3ká us3ké us3ký us1l us2lo u3so u4ste u4sty u4sté u4stě u3stř u4stš u4stý u3su. u3sy u3sá u3sí u3sů u4tro u4trá u2v u3vi u3vu u2z u3ze u3zi uz1l u3zo u3zu u3zí u2č u3ča u3če u3či u3čo uč3t u3ču u3čá u3čí u2ď u2ň u2š u3še u3ši uš4kl u3šo uš3tí u3šu u3šá u3ší u2ž u3že u3žo u3žu u3žá u3ží 1v 2v. va3dl va4jť va4kl 2v1b 2v1c v2ch 2v2d v4dal v3di v4děk v4děč ve3dle ve3jd 3ven ve2p ve3ps vep3ř ves3l ve4sm ves4p ve3sta ve3t4ř ve2z3m vi4ch vide2 vi4dr vi4et vi4kr vi2tr 2vk v2kr v2l 2v3la. 4vle. 4vlem 2vlo 2vm 2vn v4nad vo3b vo4ic vo4ja vo4jb vo4jd vo4jj vo4jm vo4jř vo2s vo4tř vou3 vous2 v2p vr2c vr2dl 4vrny v1ro vr4st vrst3v vrs4tvě 2vs2 v1sk v3stv 2v2t vy3c vy3d2 vy4dra vyp2 vy3s2 vy4sn vys4t vy3t vy3č vyč4k vyš2 vy4š. vy4šm vy4šš vy4žl v2z2 vz4no vz4né vz4ně vz4ní vá3ri 2v2č v3čá v3čí v4čír vě4cm vě3t4a více3 ví4hat 3vín 2vň 2vří v3řín v2š2 vše3s v3ští. 3výs vý3t 3vý3z v2ž2 wa4fd 3war wa4re we2 2x. xand4 2xf xisk4 2xn 3xov x1t xt4ra xy4sm y1 y2a y2bl yb3ri y2ch y4chr y2d1l yd4lá y2dr yd4y. y2e y2gr y3hn yh4ne yj4ma yj4me y2kl yk3la y3klop yk4ly ymané4 ym4kl yna4s y3ni ype4r yp4si yp4tá y2př yr2v y2s y3sa y3se y3si ys3lu y3sm y3so y3sp ys2t ys3te yst4r y3su y3sv y3sy y3sá y3sé y3sí yt4me yu3ž y3vs yvě4t y3zb y3zd y3zk y3zn yz4ně yz4ní y3zp yz4po yč2k y2ň yř3b yřk4n yř4če y3ří y2š y3še y3ši y3šk yš1l y3šo y3šp y3šu y3ší yž2 y3žd 1z 2z. zab2l za4bs za4dk za3dl za4dn za3h za3i za3j za4jk za3k za4kt zal4k zam4n za3p2 za3s2 zat2 za3tl zat4r za4ut za3z zaz4n za4zš za4č. za3š zaš4k za4šs 2zb zban4 z2by zbys4 2z1c 2z2d z3di zdně4ní z4doba z4dobný zd4re zd4ví z2e ze3h ze3p2 4zerot ze3s2 zes4p zet2 zev2 ze3vn ze3z ze4z. 2z2f z1há z4ine z2j z3jí 2z2k z3ka. z3ky z3ké z3ků z3ký 2zl 3zl. zlhos4 zlik3 z3ly. z2m2 2zme z3mn z3my z4měn 2z2n 3znak z4nal z3ne. z3nic z3no z3nu z3ny z3né z3ně z4něl z3ní z4nít z4nív z3ný zo4tr zo4šk 2z2p z3pt z4pát 3zrak 2z1s2 2zt ztros3 z4trá z3tř 3zu. zu3mo zu3mě zu3mí zu3š z2v zva4d z3vař z3vi zvik4 zv4ně z3vod z3voj z4von zv4ro z4ván z4věs z3víj 3zy. 2zz zá1 záh2 zá4kl. 3záp zá3s2 zá3z záš2 2zč z3čl 2zň z2ř zřej3 z3řez z3řeš 2zš2 z3šk zš4ka z3št 2z2ú1 zú3č zú3ž zů3s á1b á2bl áb4ry á4bř. á3cho ác3ti3 á1d á2dl ádo4s ádos4ti ád1ř á1ha á3he áh1l á3hl. áh3n á1ho á1hr á1há á1j á4jmu áj4mů á4kli ák4ni á1la á1le á1lo á1lu á1ly á3lé á1lí á3my á3mé á1mě á3mí á3mý áne4v á1ra á1re ár2m á1ro á1ru á3rů á1s á2sc á2s3k ás4k. ás4kl ás4kn á2sla ás4ly á2sm ás4po á2st át3k át1r á1tu á1ty á1tí á3tý áv4si áv4sí áz3k áz3ni ázni4c áz4vi á2ň á1ř ář4ke ář4ků á2š á3še á3ší 2č. 1ča ča4br 2čb 2č1c 1če 3če. če1c čes3k 1či 2čk č3ka. č3ko č3ku č3ky 2č1m 2čn č2ne 1čo č2p 2čs č1sk čs4la čs4sr 2č2t č4tené. č4tený čt4la č4tový. 3čtv 4čtěn č3tí 1ču 1čá 1čí čís3l 1čů 2ď. 1ďa 1ďo ďs4te 2ď1t 3ďuj é1 é2d é3di é3do é2f é3fo éf1r é2kl é2l é2m é3ma é3me é3mi é3mo é3mu é3mů 4ére. é2s é2t é3ta é3to é3tá é2š é2ž ě1c ěd3r ě3ha ě3he ě3hl. ěh3lo ěh3n ě1ho ě3hu ě3hů ě3ja ě1je ě1jo ě3jů ě4klé ě3k2t ě1l ě1ra ěra3d ě1re ě1ro ěr3s ěrs4t ě1ru ě1ry ě1rů ěs3k ěs3n ět1a3 ět4ac ět1l ě1tr ět3ra ě4traj ět3v ě1tí ět3ří ě2v ě3va ě3ve ě3vl ě3vo ě3vu ě3vá ěv3č ě2z ě3ze ě3zi ěz3n ě3zo ě3zí ě1ř ě2š ě3še ě3ši ě3šo ě3šu ě3šá ě3ší ěš3ť ěš4ťs ě2ť ě3ťo ě2ž ě3že ě3ži ě3žo ě3žu ě3ží í1b íb3ř í3cho ích4t íd1l í1h í2hl íh3n í1j íjed4 íj4mů í2kr í1l í1má í3mé í1mě í1r í1sa í2s3k ís4kl ís4kn ís4l. ís3le ís4ln ísáh2 í1t ít3k í3t3ře íz3da íz3de íz3k í3zna í3z3ni í3zněn í2ň í1ř í2š í3še í3ši í3šo í3ší 1ň 2ň. 2ňa ňa3d 2ňk 2ňm 3ňov ň1s 2ň1t ó1 ó2z ó3za ó3zi ó3zo ó3zy 2ř. řa4pl řa4ďm 2ř2b 2řc 2řd ře3ch ře4dob ře1h ře3jd ře3kl ře3kv ře4kří řeo4r ře3p2 ře4p. ře4pk ře4pč řer4v 2řes ře3ska ře3sko ře2sp řes3po ře4sr ře3sta ře3stu ře3stá ře3stř ře3tl řet4ř ře3zd ře3zk 4řezl ře3čt ři1 řia3 ři3h ři4h. ři4hn ři4jď ři4l. ři4lb řil2n 4řine řis2 3ři4t. ři4v. ři4vk ři4vn ři3z řič4t ři3ř ři4š. 2řk ř2kl řk4la řk4li řk4ly řk4no 2ř1l 2ř1m 2řn 1řo 2řou 2ř2p 2ř1s řs4to 2ř1t ř2v 2řz řá4pl řá2sl 2ř1č 2říd ří4kř ří1s 2řš ř3št řš4ti 1š 2š. šab3 ša4vl 2š1c šej4d šep3t ši4mr 2š2k š3ka š3ke š3k3li 4š3kou 4škov 3škr šk4ro š3ku. š3ky 2šl š2la š2li š3liv š2lo šlá2 š2lé š2lý 2š1m šmi4d 2šn š2p 2š1s 2št š4tip št4ka št4kl š4těk š2těs š4těv š4típ š2v ší3d š2ň š3ší 2š2ť š3ťo š3ťu š3ťá 1ť 2ť. 3ťal 2ťk 2ťm 2ťt ťáč4k 1ú ú2c2 ú2d új4ma ú2k ú2l ú2n ú2p ú2t út4ko ú2v ú2z úz3k ú2č 3úče úře4z úš4ti ú2ž ů1b ů1c ů1hl ů3jd ů4jmový ů1le ů1my ů1mě ů1ra ůr4va ůr4vy ů1s2 ů2st ůs3te ůs3tán ůt2 ů3tkl ů2v ů3va ů3vo ů3vě ů2z ů3zo ů2ž ů3že ů3ži ů3žo ý1b ý3cho ý1d ýd4la ý1h ý1j ý1l ý1ml ý1mě ý2n ý3no ýpo3č4 ý1r ý1s2 ý2sk ý1t ýt4ku ýt4ky ý1u ý4vli ý3zk ý3zn ý4zvu ýč4ně ý1ř ýš3l 1ž 2ž. ža3d ža4tv 3žač 2ž1b 2ž1c 2ž1d že2b3 žeh3n že4ml že4zg ži4dl ži4jm 3žil ži2vl 2žk žk4ni 2žl ž4lic 3žlo 2ž1m 2žn žon2 2ž1s2 2ž1t ž2v žá4br žá4nr 2žď ží4zn 2žň 2žš žš4ti žš4tě",
  ["lefthyphenmin"]=1,
  ["length"]=21736,
  ["n"]=3636,
  ["righthyphenmin"]=1,
 },
 ["version"]="1.001",
}