return {
 ["comment"]="% generated by mtxrun --script pattern --convert",
 ["exceptions"]={
  ["characters"]="abcdegijklmnoprstuvzćšабвгдезиклмнопрстуцшјћ",
  ["data"]="на-дно на-тка на-тка-ти на-тка-ше о-дно о-тка о-тка-ти о-тка-ше по-дно по-дне по-тки по-тку по-тка по-тке у-дно и-где и-гдје сву-где све-где сву-гдје све-гдје по-не-где по-не-гдје и-зби и-зба и-збе и-зби-ци и-зби-ца и-зби-це и-звит и-зим изо-ба-ри изо-ба-ру изо-ба-ра изо-ба-ре и-скок и-ско-ку и-ско-ка и-скон и-ско-ни и-ско-ну и-ско-на и-скри и-скру и-скра и-скре и-скрав и-спод и-спо-да и-стри и-стро и-стру и-стра и-стре на-ји на-јо на-ју на-ја на-је на-јам на-јест о-браз о-брет о-дви-ка о-дран о-дра-ти пре-двој ра-зми ра-змо ра-зму ра-зма ра-зме ра-зни ра-зно ра-зну ра-зна ра-зне ра-склоп ра-спе-ло ра-спе-лу ра-спе-ла ра-спе-ћу ра-спе-ћа ра-спе-ће ра-сти ра-сту ра-ста ра-сте ра-стом ра-стер ра-шћи ра-шћо ра-шћу ра-шћа ра-шће у-зни у-зно у-зна у-зник у-зрок у-ски у-ско у-ску у-ска у-ске ус-кос у-спио у-спео у-спор у-шћу у-шћа у-шће ин-те-рес тран-су тран-са тран-сом na-dno na-tka na-tka-ti na-tka-še o-dno o-tka o-tka-ti o-tka-še po-dno po-dne po-tki po-tku po-tka po-tke u-dno i-gde i-gdje svu-gde sve-gde svu-gdje sve-gdje po-ne-gde po-ne-gdje i-zbi i-zba i-zbe i-zbi-ci i-zbi-ca i-zbi-ce i-zvit i-zim izo-ba-ri izo-ba-ru izo-ba-ra izo-ba-re i-skok i-sko-ku i-sko-ka i-skon i-sko-ni i-sko-nu i-sko-na i-skri i-skru i-skra i-skre i-skrav i-spod i-spo-da i-stri i-stro i-stru i-stra i-stre na-ji na-jo na-ju na-ja na-je na-jam na-jest o-braz o-bret o-dvi-ka o-dran o-dra-ti pre-dvoj ra-zmi ra-zmo ra-zmu ra-zma ra-zme ra-zni ra-zno ra-znu ra-zna ra-zne ra-sklop ra-spe-lo ra-spe-lu ra-spe-la ra-spe-ću ra-spe-ća ra-spe-će ra-sti ra-stu ra-sta ra-ste ra-stom ra-ster ra-šći ra-šćo ra-šću ra-šća ra-šće u-zni u-zno u-zna u-znik u-zrok u-ski u-sko u-sku u-ska u-ske us-kos u-spio u-speo u-spor u-šću u-šća u-šće in-te-res tran-su tran-sa tran-som",
  ["length"]=2394,
  ["n"]=232,
 },
 ["metadata"]={
  ["mnemonic"]="sr",
  ["source"]="hyph-sr",
  ["texcomment"]="% \n% \n% \n% \n% ",
 },
 ["patterns"]={
  ["characters"]="abcdefghijklmnoprstuvzćčđšžабвгдежзиклмнопрстуфхцчшђјљњћџ",
  ["data"]="т2ј т2л т2љ т2р т2в д2ј д2л д2љ д2р д2в г2ј г2л г2љ г2р г2в х2ј х2л х2љ х2р х2в к2ј к2л к2љ к2р к2в 2д1ж 2д1б 2д1ц 2д1д 2д1ф 2д1г 2д1х 2д1к 2д1м 2д1н 2д1п 2д1с 2д1т 2д1њ 2д1џ 2д1з 2д1ш 2д1ђ 2д1ћ 2д1ч 2г1ж 2г1б 2г1ц 2г1д 2г1ф 2г1г 2г1х 2г1к 2г1м 2г1н 2г1п 2г1с 2г1т 2г1њ 2г1џ 2г1з 2г1ш 2г1ђ 2г1ћ 2г1ч 2х1ж 2х1б 2х1ц 2х1д 2х1ф 2х1г 2х1х 2х1к 2х1м 2х1н 2х1п 2х1с 2х1т 2х1њ 2х1џ 2х1з 2х1ш 2х1ђ 2х1ћ 2х1ч 2к1ж 2к1б 2к1ц 2к1д 2к1ф 2к1г 2к1х 2к1к 2к1м 2к1н 2к1п 2к1с 2к1т 2к1њ 2к1џ 2к1з 2к1ш 2к1ђ 2к1ћ 2к1ч 2т1ж 2т1б 2т1ц 2т1д 2т1ф 2т1г 2т1х 2т1к 2т1м 2т1н 2т1п 2т1с 2т1т 2т1њ 2т1џ 2т1з 2т1ш 2т1ђ 2т1ћ 2т1ч 2дј. 2дл. 2дљ. 2др. 2дв. 2гј. 2гл. 2гљ. 2гр. 2гв. 2хј. 2хл. 2хљ. 2хр. 2хв. 2кј. 2кл. 2кљ. 2кр. 2кв. 2тј. 2тл. 2тљ. 2тр. 2тв. п2ј п2л п2љ п2р в2ј в2л в2љ в2р б2ј б2л б2љ б2р ф2ј ф2л ф2љ ф2р м2ј м2л м2љ м2р 2б1ж 2б1б 2б1ц 2б1д 2б1ф 2б1г 2б1х 2б1к 2б1м 2б1н 2б1п 2б1с 2б1т 2б1в 2б1њ 2б1џ 2б1з 2б1ш 2б1ђ 2б1ћ 2б1ч 2ф1ж 2ф1б 2ф1ц 2ф1д 2ф1ф 2ф1г 2ф1х 2ф1к 2ф1м 2ф1н 2ф1п 2ф1с 2ф1т 2ф1в 2ф1њ 2ф1џ 2ф1з 2ф1ш 2ф1ђ 2ф1ћ 2ф1ч 2м1ж 2м1б 2м1ц 2м1д 2м1ф 2м1г 2м1х 2м1к 2м1м 2м1н 2м1п 2м1с 2м1т 2м1в 2м1њ 2м1џ 2м1з 2м1ш 2м1ђ 2м1ћ 2м1ч 2п1ж 2п1б 2п1ц 2п1д 2п1ф 2п1г 2п1х 2п1к 2п1м 2п1н 2п1п 2п1с 2п1т 2п1в 2п1њ 2п1џ 2п1з 2п1ш 2п1ђ 2п1ћ 2п1ч 2в1ж 2в1б 2в1ц 2в1д 2в1ф 2в1г 2в1х 2в1к 2в1м 2в1н 2в1п 2в1с 2в1т 2в1в 2в1њ 2в1џ 2в1з 2в1ш 2в1ђ 2в1ћ 2в1ч 2бј. 2бл. 2бљ. 2бр. 2фј. 2фл. 2фљ. 2фр. 2мј. 2мл. 2мљ. 2мр. 2пј. 2пл. 2пљ. 2пр. 2вј. 2вл. 2вљ. 2вр. с2ц с2ј с2к с2л с2м с2н с2п с2љ с2р с2т с2в с2њ 2с1ж 2с1б 2с1д 2с1ф 2с1г 2с1х 2с1с 2с1џ 2с1з 2с1ш 2с1ђ 2с1ћ 2с1ч 2сј. 2ск. 2сл. 2см. 2сн. 2сп. 2сљ. 2ср. 2ст. 2св. 2сњ. 2сц. з2б з2д з2г з2ј з2л з2м з2н з2љ з2р з2в з2њ 2з1ж 2з1ц 2з1ф 2з1х 2з1к 2з1п 2з1с 2з1т 2з1џ 2з1з 2з1ш 2з1ђ 2з1ћ 2з1ч 2зј. 2зл. 2зм. 2зн. 2зљ. 2зр. 2зв. 2зњ. 2зб. 2зд. 2зг. ш2ц ш2к ш2л ш2м ш2н ш2п ш2љ ш2т ш2в ш2њ ш2ћ ш2ч 2ш1ж 2ш1б 2ш1д 2ш1ф 2ш1г 2ш1х 2ш1с 2ш1џ 2ш1з 2ш1ш 2ш1ђ 2ш1ј 2ш1р 2шк. 2шл. 2шм. 2шн. 2шп. 2шљ. 2шт. 2шв. 2шњ. 2шћ. 2шч. 2шц. ж2б ж2д ж2г ж2л ж2м ж2н ж2љ ж2в ж2њ ж2ђ 2ж1ж 2ж1ц 2ж1ф 2ж1х 2ж1к 2ж1п 2ж1с 2ж1т 2ж1џ 2ж1з 2ж1ш 2ж1ћ 2ж1ч 2ж1ј 2ж1р 2жл. 2жм. 2жн. 2жљ. 2жв. 2жњ. 2жђ. 2жб. 2жд. 2жг. ц2ј ц2р ц2в 2ц1ж 2ц1б 2ц1ц 2ц1д 2ц1ф 2ц1г 2ц1х 2ц1к 2ц1л 2ц1м 2ц1н 2ц1п 2ц1љ 2ц1с 2ц1т 2ц1њ 2ц1џ 2ц1з 2ц1ш 2ц1ђ 2ц1ћ 2ц1ч 2цј. 2цр. 2цв. ч2в 2ч1ж 2ч1б 2ч1ц 2ч1д 2ч1ф 2ч1г 2ч1х 2ч1ј 2ч1к 2ч1л 2ч1м 2ч1н 2ч1п 2ч1љ 2ч1р 2ч1с 2ч1т 2ч1њ 2ч1џ 2ч1з 2ч1ш 2ч1ђ 2ч1ћ 2ч1ч 2чв. 2ј1ж 2ј1б 2ј1ц 2ј1д 2ј1ф 2ј1г 2ј1х 2ј1ј 2ј1к 2ј1л 2ј1м 2ј1н 2ј1п 2ј1љ 2ј1р 2ј1с 2ј1т 2ј1в 2ј1њ 2ј1џ 2ј1з 2ј1ш 2ј1ђ 2ј1ћ 2ј1ч 2л1ж 2л1б 2л1ц 2л1д 2л1ф 2л1г 2л1х 2л1ј 2л1к 2л1л 2л1м 2л1н 2л1п 2л1љ 2л1р 2л1с 2л1т 2л1в 2л1њ 2л1џ 2л1з 2л1ш 2л1ђ 2л1ћ 2л1ч 2н1ж 2н1б 2н1ц 2н1д 2н1ф 2н1г 2н1х 2н1ј 2н1к 2н1л 2н1м 2н1н 2н1п 2н1љ 2н1р 2н1с 2н1т 2н1в 2н1њ 2н1џ 2н1з 2н1ш 2н1ђ 2н1ћ 2н1ч 2љ1ж 2љ1б 2љ1ц 2љ1д 2љ1ф 2љ1г 2љ1х 2љ1ј 2љ1к 2љ1л 2љ1м 2љ1н 2љ1п 2љ1љ 2љ1р 2љ1с 2љ1т 2љ1в 2љ1њ 2љ1џ 2љ1з 2љ1ш 2љ1ђ 2љ1ћ 2љ1ч 2р1ж 2р1б 2р1ц 2р1д 2р1ф 2р1г 2р1х 2р1ј 2р1к 2р1л 2р1м 2р1н 2р1п 2р1љ 2р1р 2р1с 2р1т 2р1в 2р1њ 2р1џ 2р1з 2р1ш 2р1ђ 2р1ћ 2р1ч 2њ1ж 2њ1б 2њ1ц 2њ1д 2њ1ф 2њ1г 2њ1х 2њ1ј 2њ1к 2њ1л 2њ1м 2њ1н 2њ1п 2њ1љ 2њ1р 2њ1с 2њ1т 2њ1в 2њ1њ 2њ1џ 2њ1з 2њ1ш 2њ1ђ 2њ1ћ 2њ1ч 2џ1ж 2џ1б 2џ1ц 2џ1д 2џ1ф 2џ1г 2џ1х 2џ1ј 2џ1к 2џ1л 2џ1м 2џ1н 2џ1п 2џ1љ 2џ1р 2џ1с 2џ1т 2џ1в 2џ1њ 2џ1џ 2џ1з 2џ1ш 2џ1ђ 2џ1ћ 2џ1ч 2ђ1ж 2ђ1б 2ђ1ц 2ђ1д 2ђ1ф 2ђ1г 2ђ1х 2ђ1ј 2ђ1к 2ђ1л 2ђ1м 2ђ1н 2ђ1п 2ђ1љ 2ђ1р 2ђ1с 2ђ1т 2ђ1в 2ђ1њ 2ђ1џ 2ђ1з 2ђ1ш 2ђ1ђ 2ђ1ћ 2ђ1ч 2ћ1ж 2ћ1б 2ћ1ц 2ћ1д 2ћ1ф 2ћ1г 2ћ1х 2ћ1ј 2ћ1к 2ћ1л 2ћ1м 2ћ1н 2ћ1п 2ћ1љ 2ћ1р 2ћ1с 2ћ1т 2ћ1в 2ћ1њ 2ћ1џ 2ћ1з 2ћ1ш 2ћ1ђ 2ћ1ћ 2ћ1ч .х2 .ј2 .к2 .л2 .м2 .н2 .п2 .љ2 .р2 .с2 .т2 .в2 .њ2 .џ2 .з2 .ш2 .ђ2 .ћ2 .ч2 .ж2 .б2 .ц2 .д2 .ф2 .г2 о3а о3е о3и о3о о3у у3а у3е у3и у3о у3у а3а а3е а3и а3о а3у е3а е3е е3и е3о е3у и3а и3е и3и и3о и3у 2а1 2е1 2и1 2о1 2у1 2скб 2скц 2скд 2скф 2скг 2скх 2скк 2скм 2скн 2скп 2скс 2скт 2скњ 2скџ 2скз 2скш 2скђ 2скћ 2скч 2скж 2стб 2стц 2стд 2стф 2стг 2стх 2стк 2стм 2стн 2стп 2стс 2стт 2стњ 2стџ 2стз 2стш 2стђ 2стћ 2стч 2стж 2шкб 2шкц 2шкд 2шкф 2шкг 2шкх 2шкк 2шкм 2шкн 2шкп 2шкс 2шкт 2шкњ 2шкџ 2шкз 2шкш 2шкђ 2шкћ 2шкч 2шкж 2штб 2штц 2штд 2штф 2штг 2штх 2штк 2штм 2штн 2штп 2штс 2штт 2штњ 2штџ 2штз 2штш 2штђ 2штћ 2штч 2штж 2спб 2спц 2спд 2спф 2спг 2спх 2спк 2спм 2спн 2спп 2спс 2спт 2спв 2спњ 2спџ 2спз 2спш 2спђ 2спћ 2спч 2спж 2свб 2свц 2свд 2свф 2свг 2свх 2свк 2свм 2свн 2свп 2свс 2свт 2свв 2свњ 2свџ 2свз 2свш 2свђ 2свћ 2свч 2свж 2шпб 2шпц 2шпд 2шпф 2шпг 2шпх 2шпк 2шпм 2шпн 2шпп 2шпс 2шпт 2шпв 2шпњ 2шпџ 2шпз 2шпш 2шпђ 2шпћ 2шпч 2шпж 2швб 2швц 2швд 2швф 2швг 2швх 2швк 2швм 2швн 2швп 2швс 2швт 2швв 2швњ 2швџ 2швз 2швш 2швђ 2швћ 2швч 2швж 2ждб 2ждц 2ждд 2ждф 2ждг 2ждх 2ждк 2ждм 2ждн 2ждп 2ждс 2ждт 2ждњ 2ждџ 2ждз 2ждш 2ждђ 2ждћ 2ждч 2ждж 2жгб 2жгц 2жгд 2жгф 2жгг 2жгх 2жгк 2жгм 2жгн 2жгп 2жгс 2жгт 2жгњ 2жгџ 2жгз 2жгш 2жгђ 2жгћ 2жгч 2жгж 2здб 2здц 2здд 2здф 2здг 2здх 2здк 2здм 2здн 2здп 2здс 2здт 2здњ 2здџ 2здз 2здш 2здђ 2здћ 2здч 2здж 2згб 2згц 2згд 2згф 2згг 2згх 2згк 2згм 2згн 2згп 2згс 2згт 2згњ 2згџ 2згз 2згш 2згђ 2згћ 2згч 2згж 2жвб 2жвц 2жвд 2жвф 2жвг 2жвх 2жвк 2жвм 2жвн 2жвп 2жвс 2жвт 2жвв 2жвњ 2жвџ 2жвз 2жвш 2жвђ 2жвћ 2жвч 2жвж 2жбб 2жбц 2жбд 2жбф 2жбг 2жбх 2жбк 2жбм 2жбн 2жбп 2жбс 2жбт 2жбв 2жбњ 2жбџ 2жбз 2жбш 2жбђ 2жбћ 2жбч 2жбж 2звб 2звц 2звд 2звф 2звг 2звх 2звк 2звм 2звн 2звп 2звс 2звт 2звв 2звњ 2звџ 2звз 2звш 2звђ 2звћ 2звч 2звж 2збб 2збц 2збд 2збф 2збг 2збх 2збк 2збм 2збн 2збп 2збс 2збт 2збв 2збњ 2збџ 2збз 2збш 2збђ 2збћ 2збч 2збж 2жмб 2жмц 2жмд 2жмф 2жмг 2жмх 2жмк 2жмм 2жмн 2жмп 2жмс 2жмт 2жмв 2жмњ 2жмџ 2жмз 2жмш 2жмђ 2жмћ 2жмч 2жмж 2смб 2смц 2смд 2смф 2смг 2смх 2смк 2смм 2смн 2смп 2смс 2смт 2смв 2смњ 2смџ 2смз 2смш 2смђ 2смћ 2смч 2смж 2змб 2змц 2змд 2змф 2змг 2змх 2змк 2змм 2змн 2змп 2змс 2змт 2змв 2змњ 2змџ 2змз 2змш 2змђ 2змћ 2змч 2змж 2шмб 2шмц 2шмд 2шмф 2шмг 2шмх 2шмк 2шмм 2шмн 2шмп 2шмс 2шмт 2шмв 2шмњ 2шмџ 2шмз 2шмш 2шмђ 2шмћ 2шмч 2шмж 2сцб 2сцц 2сцд 2сцф 2сцг 2сцх 2сцк 2сцл 2сцм 2сцн 2сцп 2сцљ 2сцр 2сцс 2сцт 2сцњ 2сцџ 2сцз 2сцш 2сцђ 2сцћ 2сцч 2сцж 2шцб 2шцц 2шцд 2шцф 2шцг 2шцх 2шцк 2шцл 2шцм 2шцн 2шцп 2шцљ 2шцр 2шцс 2шцт 2шцњ 2шцџ 2шцз 2шцш 2шцђ 2шцћ 2шцч 2шцж 2шчб 2шчц 2шчд 2шчф 2шчг 2шчх 2шчј 2шчк 2шчл 2шчм 2шчн 2шчп 2шчљ 2шчр 2шчс 2шчт 2шчњ 2шчџ 2шчз 2шчш 2шчђ 2шчћ 2шчч 2шчж 2хвб 2хвц 2хвд 2хвф 2хвг 2хвх 2хвк 2хвм 2хвн 2хвп 2хвс 2хвт 2хвњ 2хвџ 2хвз 2хвш 2хвђ 2хвћ 2хвч 2хвж 2ж3вл 2ж3вљ 2ц3вл 2ц3вљ 2з3вл 2з3вљ 2ш3вл 2ш3вљ 2ч3вл 2ч3вљ 2ч3вј 2с3вљ 2д3вл 2д3вљ 2д3вр 2к3вј 2к3вл 2к3вљ 2т3вј 2т3вл 2т3вљ 2г3вј 2г3вл 2г3вљ 2г3вр 2х3вј 2х3вл 2х3вљ 2х3вр 2ж3мј 2ж3мл 2ж3мљ 2ж3мр 2з3мл 2з3мр 2ш3мј 2ш3мл 2ш3мљ 2ш3цј 2ш3цв 2ш3чв 2ш3тј 2ш3тл 2ш3тљ 2с3тл 2с3кј 2с3кљ 2ш3пј 2ш3пл 2ш3пљ 2ж3дј 2ж3дл 2ж3дљ 2ж3дв 2ж3гј 2ж3гл 2ж3гљ 2ж3гр 2ж3гв 2з3дл 2з3дљ 2з3дв 2з3гј 2з3гљ 2ж3бј 2ж3бл 2ж3бљ 2ж3бр 2з3бљ .а4е2ро .бе4о .би4о .ге4о .за3г2н .за3т2ка .иза3г2н .иза3т2к .из3г2н .изд2но .изд2ну .изд2на .изр2к .ист2к .наг2н .наг2њ .на3д2нев .на3д2нич .на3д2ниц .на3т2ках .на3т2кам .на3т2касм .на3т2каст .ода3г2н .ода3д2н .од3г2н .од3м2н .о3т2ках .о3т2кам .о3т2касм .о3т2каст .по3г2н .по3д2нев .по3м2н .по3м2њ .по3р2в .по3р2ђ .по3т2ках .по3т2кам .по3т2кат .по3т2кав .пред3м2н .пред3м2њ .пре3т2ках .пре3т2кам .пре3т2кат .про3г2н .про3т2ки .про3т2ка .раза3г2н .раз3г2н .раз3д2ни .раза3т2ка .у3г2ми .у3г2н .уза3т2ка 3х2тети 3х2тјети 3х2тел 3х2тев 3х2тењ 3х2тјел 3х2тјев 3х2тјењ 3г2дегод. 3г2дјегод. 3г2декак 3г2декад 3г2дјекак 3г2дјекад ни3г2де. не3г2де. ни3г2дје. не3г2дје. 3б2дет 3б2дењ 3б2дјет 3б2дјењ 3г2мил 3г2миљ 3г2миз 3г2нос 3г2ноз 3г2ној 3г2нај 3г2незд 3г2нијезд 3г2нежђ 3г2нијежђ 3г2нев 3г2њев 3г2њав 3г2њес 3г2њет 3г2њеч 3г2њил 3г2њио 3г2њиљ 3г2њит 3г2њур 3к2нез 3к2неж 3к2њиж 3к2њиг 3м2нож 3м2ног 3м2наж 3п2сик 3п2сич 3п2сов 3п2суј 3р2ђа 3с2фер 3т2маст 3т2мул 3т2муо 3т2муљ 3т2мур 3ц2миз 3ц2мак 3ц2мач 3ц2мок 3ч2лан 3ч2лањ 3р2је 4р3јем 4р3је. .бе2з3ј .бе2з3л .бе2з3м .бе2з3н .бе2з3љ .бе2з3р .бе2з3в .бе2з3њ .бе2з3б .бе2з3д .бе2з3г .бе2з3и .бе2з3о .бе2з3у .бе2з3алкохол .бе2з3атомск .бе3з4бедн .бе3з4беда .бе3з4бједн .бе3з4бједа .бе3з4бели .бе3з4бол .бе3з4вучн .бе3з4вуча .бе3з4истан .бе3з4истен .бе3з4јак .бе3з4јач .бе3з4лобн .бе3з4лоба .бе3з4начај .бе3з4рачн .бе3з4рача .бе3з4уп .бе3з4уб .бе2с3ц .бе2с3к .бе2с3п .бе2с3т .бе3с4крупул .бе3с4покојн .бе3с4покоја .бе3с4порн .бе3с4пора .бе3с4твар .бе3с4тид .бе3с4тија .бе3с4тилу .бе3с4тиљ .бе3с4трана .бе3с4трас .бес4тселер .бе2ш3ћ .бе2ш3ч .ва2н3евр .ва2н3устав .и2з3б .и2з3д .и2з3г .и2з3ј .и2з3л .и2з3м .и2з3н .и2з3љ .и2з3р .и2з3в .и2з3њ .и2з3и .и2з3о .и2з3у .и2з3бија .и2з3бива .и2з3веди .и2з3ведн .и2з3ведб .и2з3веде .и2з3дај .и2з3аба .и2з3ака .и2з3анал .и3з4бав .и3з4бичкава .и3з4блеушан .и3з4бојак .и3з4бојк .и3з4вали .и3з4валу .и3з4вала .и3з4вале .и3з4ваљи .и3з4вижд .и3з4виискр .и3з4вија .и3з4вијен .и3з4вин .и3з4вир .и3з4вињ .и3з4витоп .и3з4вјед .и3з4војац .и3з4војц .и3з4вор .и3з4гомет .и3з4гред .и3з4грн .и3з4грт .и3з4драв .и3з4иђ .и3з4ид .и3з4ими .и3з4јежљ .и3з4лоз .и3з4лож .и3з4лог .и3з4лопаћ .и3з4ним .и3з4ној .из4оанем .из4оаном .из4обат .из4оброн .из4огам .из4огео .из4оглос .из4огон .из4ограф .из4одим .из4один .из4одоз .из4оклин .из4околон .и3з4олат .и3з4олац .и3з4олир .и3з4олов .из4олекс .из4олукс .из4омер .из4ометр .из4оморф .из4онеф .из4оном .из4опат .из4опер .из4опле .из4опол .из4опсеф .из4орах .из4осеи .из4осинт .из4осист .из4оскел .из4оскоп .из4остаз .из4осте .из4отах .из4отал .из4отер .из4отон .из4отоп .из4отро .из4офон .из4офот .из4охал .из4охаз .из4охел .из4охиј .из4охим .из4охит .из4охипс .из4охор .из4охро .и3з4раел .и3з4раиљ .и3з4рачи .и3з4ун .и3з4упч .и2с3ц .и2с3к .и2с3п .и2с3т .и3с4как .и3с4кат .и3с4кањ .и3с4кариот .и3с4квас .и3с4кврч .и3с4кин .и3с4кита .и3с4конск .и3с4коч .и3с4крам .и3с4крит .и3с4криш .и3с4крич .и3с4криц .и3с4крат .и3с4крен .и3с4крењ .и3с4крој .и3с4крсн .и3с4крса .и3с4купља .и3с4лам .и3с4лаб .и3с4леђ .и3с4лед .и3с4лијеђ .и3с4лијед .и3с4љеђ .и3с4љед .и3с4лик .и3с4лин .и3с4лов .и3с4луш .и3с4луж .и3с4ме .и3с4мије .и3с4мје .и3с4пав .и3с4паљив .и3с4пира .и3с4плит .и3с4плић .и3с4покој .и3с4полин .и3с4пон .и3с4порав .и3с4прави .и3с4правк .и3с4правн .и3с4прављ .и3с4права .и3с4пупч .и3с4пур .и3с4ред .и3с4рк .и3с4тави .и3с4тављ .и3с4такн .и3с4там .и3с4тар .и3с4тас .и3с4таћ .и3с4тин .и3с4тир .и3с4тиц .и3с4тифан .и3с4ток .и3с4тори .и3с4точн .и3с4точњ .и3с4точа .и3с4трав .и3с4трад .и3с4тран .и3с4трић .и3с4триж .и3с4триц .и3с4труг .и3с4туп .и3с4ук .и3с4ус .и3с4ут .и3с4уш .и2ж3ђ .и2ш3ћ .и2ш3ч .из3бе2з3обр .из3бе2з3ум .из3ва2н3евр .на2д3л .на2д3љ .на2д3в .на3д4вал .на3д4веси .на3д4вест .на3д4виј .на3д4вит .на3д4вла .на3д4воје .на3д4вор .на2д3игр .на2д3инж .на2д3ина .на2д3иск .на2д3јах .на2д3јач .на2д3јек .на2д3јез .на2д3јеч .на2д3јун .на3д4лан .на3д4леш .на3д4леж .на2д3ора .на2д3осо .на2д3осе .на2д3осје .на2д3офи .на2д3оч .на2д3ран .на2д3рач .на2д3раст .на2д3рашћ .на2д3реал .на2д3реп .на2д3рук .на2д3руч .на2д3руг .на2д3удар .на2д3ум .на2д3уч .на2ј3а .на2ј3е .на2ј3и .на2ј3о .на2ј3у .на3ј4ави .на3ј4ављ .на3ј4ава .на3ј4аве .на3ј4ади .на3ј4ада .на3ј4аде .на3ј4ажи .на3ј4ази .на3ј4ако .на3ј4ака .на3ј4ало .на3ј4ами .на3ј4амл .на3ј4амн .на3ј4ари .на3ј4арм .на3ј4арц .на3ј4ати .на3ј4аук .на3ј4ах .на3ј4аш .на3ј4еди .на3ј4едн .на3ј4едр .на3ј4еда .на3ј4ежи .на3ј4ежу .на3ј4еже .на3ј4езн .на3ј4езд .на3ј4ести .на3ј4етк .на3ј4ец .на3ј4ури .на3ј4урен .о2б3ј .о2б3љ .о2б3р .обе2з3б .обе2з3д .обе2з3г .обе2з3ј .обе2з3л .обе2з3м .обе2з3н .обе2з3о .обе2з3љ .обе2з3р .обе2з3у .обе2з3в .обе3з4виј .обе3з4нан .обе3з4нањ .обе3з4нач .обе3з4уб .обе2с3ц .обе2с3к .обе2с3п .обе2с3т .обе3с4тан .обе3с4тиј .обе3с4тран .обе2ш3ћ .обе2ш3ч .о2б3игр .о2б3истин .о2б3истињ .о3б4јек .о3б4јер .о3б4јеси .о3б4јет .о3б4јеш .о2б3лај .о2б3лам .о2б3лакш .о2б3ласк .о2б3леп .о2б3лет .о2б3лећ .о2б3леж .о2б3лег .о2б3лијеп .о2б3лијет .о2б3лијеж .о2б3лијег .о2б3леден .о2б3лив .о2б3лизат .о2б3лизав .о2б3лизи .о2б3лист .о2б3лока .о2б3лук .о2б3луч .о3б4љан .о3б4љут .о3б4љуз .о2б3ору .о3б4раже .о3б4рази .о3б4разн .о3б4разо .о3б4разу .о3б4раза .о3б4разд .о3б4рамб .о3б4ран .о3б4рањ .о3б4рат .о3б4раћ .о3б4рашн .о3б4рашч .о3б4рв .о3б4рђ .о3б4рем .о3б4рес .о3б4ређ .о3б4реч .о3б4реж .о3б4рец .о3б4ред .о3б4рети .о3б4ретн .о3б4риј .о3б4рис .о3б4рит .о3б4рив .о3б4рич .о3б4риц .о3б4рк .о3б4рл .о3б4рн .о3б4рљ .о3б4рс .о3б4рт .о3б4рш .о3б4рч .о3б4рок .о3б4рон .о3б4роњ .о3б4роћ .о3б4роч .о3б4рова .о3б4ровц .о3б4рук .о3б4рун .о3б4рус .о3б4руњ .о3б4руш .о3б4руч .о2б3убож .о2б3уз .о2б3уж .о2б3уд .о2б3уми .о2б3умј .о2б3умр .о2б3уме .о2д3ј .о2д3л .о2д3љ .о2д3р .о2д3в .о2д3арг .о3д4вај .о3д4важ .о3д4весн .о3д4вест .о3д4веса .о3д4викав .о3д4викн .о3д4вис .о3д4вић .о3д4вој .о2д3игр .о2д3и2з3в .о2д3и2з3д .о2д3иск .о2д3исти .о3д4јел .о3д4јен .о3д4јев .о3д4јећ .о3д4лаз .о3д4лаж .о3д4лаг .о3д4лака .о3д4лук .о3д4луч .о2д3озд .о2д3озг .о2д3ок .о2д3онл .о2д3оно .о2д3ону .о2д3онд .о3д4рани .о3д4рано .о3д4рану .о3д4рана .о3д4ране .о3д4раз .о3д4раћ .о3д4раж .о3д4рапи .о3д4рапљ .о3д4рапа .о3д4рачи .о3д4рвен .о3д4рвењ .о3д4рвеч .о3д4рем .о3д4рен .о3д4рет .о3д4ређ .о3д4ред .о3д4рл .о3д4рн .о3д4рп .о3д4рљ .о3д4рт .о3д4рж .о3д4рин .о3д4рињ .о3д4риш .о3д4рич .о3д4риб .о3д4риц .о3д4рон .о3д4роњ .о3д4руж .о3д4руг .о2д3уви .о2д3уве .о2д3узи .о2д3узл .о2д3узд .о2д3узе .о2д3ук .о2д3ул .о2д3ум .о2д3уч .по2д3адм .по2д3вариј .по2д3вез .по2д3веч .по2д3веж .по2д3вик .по2д3вил .по2д3вир .по2д3вињ .по2д3влас .по2д3влаш .по2д3воз .по2д3вођ .по2д3вож .по2д3вод .по2д3врат .по2д3враћ .по2д3врћ .по2д3врж .по2д3врг .по2д3врис .по2д3врс .по2д3вућ .по2д3игр .по2д3изв .по2д3ј .по3д4јен .по3д4јеч .по2д3лакат .по2д3лакт .по2д3леп .по2д3лет .по2д3лећ .по2д3леж .по2д3лег .по2д3лиз .по2д3лијеп .по2д3лијет .по2д3лијећ .по2д3лијеж .по2д3лијег .по2д3лист .по2д3лок .по2д3лом .по2д3луп .по2д3луч .по2д3луж .по2д3љут .по2д3окн .по2д3ош .по2д3оч .по2д3оф .по2д3равн .по2д3равњ .по2д3рад .по2д3ра2з3д .по2д3разр .по2д3разу .по2д3рам .по2д3ран .по2д3рас .по2д3рањ .по2д3реп .по2д3рес .по2д3рез .по2д3рик .по2д3рит .по2д3рон .по2д3ров .по2д3рож .по2д3рук .по2д3руб .по2д3ручи .по2д3ручн .по2д3руча .по2д3упла .по2д3усм .по2д3усн .пре2д3ј .пре2д3в .пре3д4вај .пре3д4вар .пре3д4вест .пре3д4воји .пре3д4воја .пре3д4воје .пре3д4вор .пре3д4вос .пре3д4јен .пре2д3игр .пре2д3ид .пре2д3изб .пре2д3испи .пре2д3исто .пре2д3истр .пре2д3обј .пре2д3одре .пре2д3окус .пре2д3осв .пре2д3осе .пре2д3осје .пре2д3рат .пре2д3рач .пре2д3рад .пре2д3руч .пре2д3убеђ .пре2д3убијеђ .пре2д3убјеђ .пре2д3увер .пре2д3увјер .пре2д3увјет .пре2д3угов .пре2д3удар .пре2д3упис .пре2д3усло .проти2в3акц .проти2в3отр .проти2в3оф .проти2в3р .проти2в3ус .проти2в3уд .ра2ж3ђ .ра2з3б .ра2з3д .ра2з3е .ра2з3г .ра2з3и .ра2з3ј .ра2з3л .ра2з3м .ра2з3н .ра2з3љ .ра2з3р .ра2з3в .ра2з3њ .ра2з3анал .ра3з4бан .ра3з4бар .ра3з4бау .ра3з4бад .ра3з4башур .ра3з4бој .ра3з4бор .ра3з4вал .ра3з4веде .ра3з4вест .ра3з4виго .ра3з4вију .ра3з4вија .ра3з4вије .ра3з4вит .ра3з4вић .ра3з4вој .ра3з4вон .ра3з4врат .ра3з4враћ .ра3з4врт .ра3з4врћ .ра3з4гађ .ра3з4грт .ра3з4ев .ра3з4иј .ра3з4ил .ра3з4ин .ра3з4ир .ра3з4ит .ра3з4из .ра3з4иђ .ра3з4ић .ра3з4ид .ра3з4лаз .ра3з4лаг .ра3з4лик .ра3з4лич .ра3з4лоз .ра3з4лож .ра3з4лог .ра3з4мет .ра3з4мећ .ра3з4мрск .ра3з4нат .ра2з3обл .ра2з3обр .ра2з3оба .ра2з3од .ра2з3орат .ра2з3орав .ра2з3орт .ра2з3ору .ра2з3от .ра3з4ред .ра3з4рок .ра3з4роч .ра2з3уве .ра2з3уди .ра2з3уда .ра2з3удб .ра2з3узи .ра2з3узд .ра2з3узе .ра2з3улар .ра2з3умр .ра2с3ц .ра2с3к .ра2с3п .ра2с3т .ра3с4как .ра3с4канд .ра3с4кин .ра3с4клап .ра3с4клањ .ра3с4клад .ра3с4клон .ра3с4клопи .ра3с4клопљ .ра3с4клопа .ра3с4кош .ра3с4кроп .ра3с4пај .ра3с4пав .ра3с4пети .ра3с4пето .ра3с4пета .ра3с4пете .ра3с4пику .ра3с4пињ .ра3с4плин .ра3с4плињ .ра3с4пн .ра3с4полож .ра3с4пон .ра3с4пор .ра3с4прав .ра3с4прем .ра3с4рђ .ра3с4рд .ра3с4ре .ра3с4тај .ра3с4тан .ра3с4тат .ра3с4тав .ра3с4тењ .ра3с4тил .ра3с4тир .ра3с4тис .ра3с4тит .ра3с4тињ .ра3с4тој .ра3с4трел .ра3с4трет .ра3с4трој .ра3с4трт .ра3с4туп .ра3с4тур .ра3с4тућ .ра4с5турч .ра2ш3ћ .ра2ш3ч .ра3ш4ћењ .ра3ш4чић .у2з3б .у2з3д .у2з3г .у2з3и .у2з3ј .у2з3л .у2з3м .у2з3н .у2з3љ .у2з3р .у2з3в .у2з3њ .у3з4бор .у3з4ван .у3з4ват .у3з4виж .у3з4вијо .у3з4вију .у3з4вија .у3з4вије .у3з4вој .у3з4диц .у2з3игр .у2з3инат .у2з3искр .у3з4лан .у3з4лат .у3з4лим .у3з4лит .у3з4лић .у3з4лиц .у3з4лов .у3з4лудоб .у3з4нак .у3з4нач .у3з4неве .у3з4невје .у3з4нич .у3з4ниц .у3з4ној .у2з3обест .у2з3обијест .у2з3орат .у2з3орав .у2з3охо .у3з4рет .у3з4рев .у3з4ријет .у3з4ријев .у3з4рн .у3з4рњ .у3з4роко .у3з4року .у3з4рока .у3з4роч .у3з4руј .у2з3угар .у2с3ц .у2с3к .у2с3п .у3с4как .у3с4клађ .у3с4клад .у3с4ко .у4с5ком .у4с5ков .у4с5кош .у4с5коко .у4с5колу .у4с5коле .у4с5копа .у4с5кора .у4с5коси .у4с5котр .у3с4куп .у3с4пав .у3с4пало .у3с4пех .у3с4пел .у3с4пем .у3с4пет .у3с4пев .у3с4пеш .у3с4пјех .у3с4пјел .у3с4пјем .у3с4пјет .у3с4пјев .у3с4пјеш .у3с4пент .у3с4пија .у3с4пије .у3с4пијуш .у3с4пикуш .у3с4пон .у3с4пори .у3с4пора .у3с4порен .у3с4порењ .у3с4пореч .у3с4пособ .у3с4преми .у3с4према .у3с4рк .у3с4рн .у3с4рп .у3с4рљ .у3с4рт .у3с4рђ .у3с4рж .у3с4ра .у3с4рд .у3с4ре .у3с4ријед .у2с3талас .у2с3тара .у2с3тврђ .у2с3тврд .у2с3тер .у2с3тећ .у2с3тег .у2с3тов .у2с3трај .у2с3трал .у2с3трг .у2с3треп .у2с3трес .у2с3треб .у2с3трк .у2с3трн .у2с3трп .у2с3трћ .у2с3трч .у2с3тум .у2с3тур .у2с3тућ .у2ш3ћ .у2ш3ч .а2б3алиј .а2б3анац .а2б3евак .а2б3ерац .а2б3ерир .а2б3ирит .а2б3ју .а2б3ла .а2б3лег .а2б3леп .а2б3лок .а2б3лу .а2б3ориг .а2б3реак .а2б3рог .а2б3узус .а2д3ерац .а2д3верб .а2д3ј .а2д3лат .а2д3рен .а2д3рог .а3г2нос .а3г2ноз .а2набап .а2набаз .а2набат .а2набио .а2набол .а2наген .а2нагно .а2н3аго .а2награ .а2надем .а2надипл .а2надоз .а2н3а4е2ро .а2накал .а2накам .а2накат .а2накеф .а2накла .а2накли .а2накој .а2н3акуз .а2н3алг .а2н3алд .а2налеп .а2нализ .а2налис .а2налит .а2н3амерт .а2намне .а2н3андр .а2нанео .а2н3ант .а2напла .а2напле .а2напне .а2напно .а2напро .а2напти .а2н3апто .а2нарт .а2н3арх .а2насар .а2насеи .а2наспаз .а2наста .а2настиг .а2настом .а2натим .а2натом .а2натоц .а2натре .а2натри .а2натро .а2нафаз .а2н3афиј .а2нафила .а2нафон .а2н3афрод .а2накол .а2накрон .а2накру .а2н3алфа .а2нафор .а2нахор .а2нахро .а2н3егер .а2н3екл .а2н3екум .а2н3елек .а2н3енер .а2н3епи .а2неор .а2н3ерг .а2н3ерит .а2н3есте .а2н3идр .а2н3изог .а2н3изом .а2н3изур .а2н3ирид .а2н3овар .а2н3окс .а2н3опис .а2н3орх .а2н3офт .а2н3орг .ди2с3акор .ди2с3јунк .ди2с3квал .ди2с3конт .ди2с3корд .ди2с3кре .ди2с3кри .ди2с3кур .ди2с3ло .ди2с3ориј .ди2с3парит .ди2с3поз .ди2с3пон .ди2с3проп .ди2с3тон .ди2с3трак .и2н3абруп .и2н3адек .и2н3акур .и2н3акце .и2н3амор .и2н3аниц .и2н3аплик .и2н3апст .и2н3арт .и2н3аугур .и2н3аура .и2н3афек .и2н3евид .и2н3ег .и2н3ед .и2н3екв .и2н3екс .и2н3елиг .и2н3епц .и2н3ефек .и2н3обл .и2ноген .и2нокор .и2н3окуп .и2н3опер .и2н3опор .и2н3опсе .и2н3офиц .и2н3умбр .и2н3унда .и2н3унк .и2н3утил .инте2р3и .инте2р3о .инте2р3у .инте2р3а .инте2р3е .инте3р4егн .инте3р4еси .инте3р4есн .инте3р4есо .инте3р4есу .инте3р4еса .инте3р4есе .инте3р4ежџ .инте3р4ије .инте2р3је .инте3р4огат .јури2с3к .јури2с3п .ну2з3бел .ну2з3биљ .ну2з3љуб .ну2з3ре .ну2з3рје .ну2з3уж .ну2с3пос .ну2с3про .по2ст3егз .по2ст3инду .по2ст3лим .по2ст3онк .по2ст3опер .су2б3а .су2б3л .су3б4аш .су2б3инв .су2б3јунк .су2б3окс .су2б3реп .су2б3рог .су2б3орд .супе2р3и .супе2р3о .супе2р3у .супе2р3а .супе2р3е .супе3р4иор .тран2с3а .тран2с3ц .тран2с3е .тран2с3к .тран2с3л .тран2с3м .тран2с3н .тран2с3о .тран2с3п .тран2с3т .тран2с3у .тран2с3в .тран2с3њ .тран3с4еп .тран3с4кри .тран3с4ум .тран3с4уд t2j t2l t2lj t2r t2v d2j d2l d2lj d2r d2v g2j g2l g2lj g2r g2v h2j h2l h2lj h2r h2v k2j k2l k2lj k2r k2v 2d1b 2d1c 2d1d 2d1f 2d1g 2d1h 2d1k 2d1m 2d1n 2d1p 2d1s 2d1t 2d1nj 2d1dž 2d1z 2d1š 2d1đ 2d1ć 2d1č 2g1ž 2g1b 2g1c 2g1d 2g1f 2g1g 2g1h 2g1k 2g1m 2g1n 2g1p 2g1s 2g1t 2g1nj 2g1dž 2g1z 2g1š 2g1đ 2g1ć 2g1č 2h1ž 2h1b 2h1c 2h1d 2h1f 2h1g 2h1h 2h1k 2h1m 2h1n 2h1p 2h1s 2h1t 2h1nj 2h1dž 2h1z 2h1š 2h1đ 2h1ć 2h1č 2k1ž 2k1b 2k1c 2k1d 2k1f 2k1g 2k1h 2k1k 2k1m 2k1n 2k1p 2k1s 2k1t 2k1nj 2k1dž 2k1z 2k1š 2k1đ 2k1ć 2k1č 2t1ž 2t1b 2t1c 2t1d 2t1f 2t1g 2t1h 2t1k 2t1m 2t1n 2t1p 2t1s 2t1t 2t1nj 2t1dž 2t1z 2t1š 2t1đ 2t1ć 2t1č 2dj. 2dl. 2dlj. 2dr. 2dv. 2gj. 2gl. 2glj. 2gr. 2gv. 2hj. 2hl. 2hlj. 2hr. 2hv. 2kj. 2kl. 2klj. 2kr. 2kv. 2tj. 2tl. 2tlj. 2tr. 2tv. p2j p2l p2lj p2r v2j v2l v2lj v2r b2j b2l b2lj b2r f2j f2l f2lj f2r m2j m2l m2lj m2r 2b1ž 2b1b 2b1c 2b1d 2b1f 2b1g 2b1h 2b1k 2b1m 2b1n 2b1p 2b1s 2b1t 2b1v 2b1nj 2b1dž 2b1z 2b1š 2b1đ 2b1ć 2b1č 2f1ž 2f1b 2f1c 2f1d 2f1f 2f1g 2f1h 2f1k 2f1m 2f1n 2f1p 2f1s 2f1t 2f1v 2f1nj 2f1dž 2f1z 2f1š 2f1đ 2f1ć 2f1č 2m1ž 2m1b 2m1c 2m1d 2m1f 2m1g 2m1h 2m1k 2m1m 2m1n 2m1p 2m1s 2m1t 2m1v 2m1nj 2m1dž 2m1z 2m1š 2m1đ 2m1ć 2m1č 2p1ž 2p1b 2p1c 2p1d 2p1f 2p1g 2p1h 2p1k 2p1m 2p1n 2p1p 2p1s 2p1t 2p1v 2p1nj 2p1dž 2p1z 2p1š 2p1đ 2p1ć 2p1č 2v1ž 2v1b 2v1c 2v1d 2v1f 2v1g 2v1h 2v1k 2v1m 2v1n 2v1p 2v1s 2v1t 2v1v 2v1nj 2v1dž 2v1z 2v1š 2v1đ 2v1ć 2v1č 2bj. 2bl. 2blj. 2br. 2fj. 2fl. 2flj. 2fr. 2mj. 2ml. 2mlj. 2mr. 2pj. 2pl. 2plj. 2pr. 2vj. 2vl. 2vlj. 2vr. s2c s2j s2k s2l s2m s2n s2p s2lj s2r s2t s2v s2nj 2s1ž 2s1b 2s1d 2s1f 2s1g 2s1h 2s1s 2s1dž 2s1z 2s1š 2s1đ 2s1ć 2s1č 2sj. 2sk. 2sl. 2sm. 2sn. 2sp. 2slj. 2sr. 2st. 2sv. 2snj. 2sc. z2b z2d z2g z2j z2l z2m z2n z2lj z2r z2v z2nj 2z1ž 2z1c 2z1f 2z1h 2z1k 2z1p 2z1s 2z1t 2z1dž 2z1z 2z1š 2z1đ 2z1ć 2z1č 2zj. 2zl. 2zm. 2zn. 2zlj. 2zr. 2zv. 2znj. 2zb. 2zd. 2zg. š2c š2k š2l š2m š2n š2p š2lj š2t š2v š2nj š2ć š2č 2š1ž 2š1b 2š1d 2š1f 2š1g 2š1h 2š1s 2š1dž 2š1z 2š1š 2š1đ 2š1j 2š1r 2šk. 2šl. 2šm. 2šn. 2šp. 2šlj. 2št. 2šv. 2šnj. 2šć. 2šč. 2šc. ž2b ž2d ž2g ž2l ž2m ž2n ž2lj ž2v ž2nj ž2đ 2ž1ž 2ž1c 2ž1f 2ž1h 2ž1k 2ž1p 2ž1s 2ž1t 2ž1dž 2ž1z 2ž1š 2ž1ć 2ž1č 2ž1j 2ž1r 2žl. 2žm. 2žn. 2žlj. 2žv. 2žnj. 2žđ. 2žb. 2žd. 2žg. c2j c2r c2v 2c1ž 2c1b 2c1c 2c1d 2c1f 2c1g 2c1h 2c1k 2c1l 2c1m 2c1n 2c1p 2c1lj 2c1s 2c1t 2c1nj 2c1dž 2c1z 2c1š 2c1đ 2c1ć 2c1č 2cj. 2cr. 2cv. č2v 2č1ž 2č1b 2č1c 2č1d 2č1f 2č1g 2č1h 2č1j 2č1k 2č1l 2č1m 2č1n 2č1p 2č1lj 2č1r 2č1s 2č1t 2č1nj 2č1dž 2č1z 2č1š 2č1đ 2č1ć 2č1č 2čv. 2j1ž 2j1b 2j1c 2j1d 2j1f 2j1g 2j1h 2j1j 2j1k 2j1l 2j1m 2j1n 2j1p 2j1lj 2j1r 2j1s 2j1t 2j1v 2j1nj 2j1dž 2j1z 2j1š 2j1đ 2j1ć 2j1č 2l1ž 2l1b 2l1c 2l1d 2l1f 2l1g 2l1h 2l1k 2l1l 2l1m 2l1n 2l1p 2l1lj 2l1r 2l1s 2l1t 2l1v 2l1nj 2l1dž 2l1z 2l1š 2l1đ 2l1ć 2l1č 2n1ž 2n1b 2n1c 2n1d 2n1f 2n1g 2n1h 2n1k 2n1l 2n1m 2n1n 2n1p 2n1lj 2n1r 2n1s 2n1t 2n1v 2n1nj 2n1dž 2n1z 2n1š 2n1đ 2n1ć 2n1č 2lj1ž 2lj1b 2lj1c 2lj1d 2lj1f 2lj1g 2lj1h 2lj1j 2lj1k 2lj1l 2lj1m 2lj1n 2lj1p 2lj1lj 2lj1r 2lj1s 2lj1t 2lj1v 2lj1nj 2lj1dž 2lj1z 2lj1š 2lj1đ 2lj1ć 2lj1č 2r1ž 2r1b 2r1c 2r1d 2r1f 2r1g 2r1h 2r1j 2r1k 2r1l 2r1m 2r1n 2r1p 2r1lj 2r1r 2r1s 2r1t 2r1v 2r1nj 2r1dž 2r1z 2r1š 2r1đ 2r1ć 2r1č 2nj1ž 2nj1b 2nj1c 2nj1d 2nj1f 2nj1g 2nj1h 2nj1j 2nj1k 2nj1l 2nj1m 2nj1n 2nj1p 2nj1lj 2nj1r 2nj1s 2nj1t 2nj1v 2nj1nj 2nj1dž 2nj1z 2nj1š 2nj1đ 2nj1ć 2nj1č 2dž1ž 2dž1b 2dž1c 2dž1d 2dž1f 2dž1g 2dž1h 2dž1j 2dž1k 2dž1l 2dž1m 2dž1n 2dž1p 2dž1lj 2dž1r 2dž1s 2dž1t 2dž1v 2dž1nj 2dž1dž 2dž1z 2dž1š 2dž1đ 2dž1ć 2dž1č 2đ1ž 2đ1b 2đ1c 2đ1d 2đ1f 2đ1g 2đ1h 2đ1j 2đ1k 2đ1l 2đ1m 2đ1n 2đ1p 2đ1lj 2đ1r 2đ1s 2đ1t 2đ1v 2đ1nj 2đ1dž 2đ1z 2đ1š 2đ1đ 2đ1ć 2đ1č 2ć1ž 2ć1b 2ć1c 2ć1d 2ć1f 2ć1g 2ć1h 2ć1j 2ć1k 2ć1l 2ć1m 2ć1n 2ć1p 2ć1lj 2ć1r 2ć1s 2ć1t 2ć1v 2ć1nj 2ć1dž 2ć1z 2ć1š 2ć1đ 2ć1ć 2ć1č .h2 .j2 .k2 .l2 .m2 .n2 .p2 .lj2 .r2 .s2 .t2 .v2 .nj2 .dž2 .z2 .š2 .đ2 .ć2 .č2 .ž2 .b2 .c2 .d2 .f2 .g2 o3a o3e o3i o3o o3u u3a u3e u3i u3o u3u a3a a3e a3i a3o a3u e3a e3e e3i e3o e3u i3a i3e i3i i3o i3u 2a1 2e1 2i1 2o1 2u1 2skb 2skc 2skd 2skf 2skg 2skh 2skk 2skm 2skn 2skp 2sks 2skt 2sknj 2skdž 2skz 2skš 2skđ 2skć 2skč 2skž 2stb 2stc 2std 2stf 2stg 2sth 2stk 2stm 2stn 2stp 2sts 2stt 2stnj 2stdž 2stz 2stš 2stđ 2stć 2stč 2stž 2škb 2škc 2škd 2škf 2škg 2škh 2škk 2škm 2škn 2škp 2šks 2škt 2šknj 2škdž 2škz 2škš 2škđ 2škć 2škč 2škž 2štb 2štc 2štd 2štf 2štg 2šth 2štk 2štm 2štn 2štp 2šts 2štt 2štnj 2štdž 2štz 2štš 2štđ 2štć 2štč 2štž 2spb 2spc 2spd 2spf 2spg 2sph 2spk 2spm 2spn 2spp 2sps 2spt 2spv 2spnj 2spdž 2spz 2spš 2spđ 2spć 2spč 2spž 2svb 2svc 2svd 2svf 2svg 2svh 2svk 2svm 2svn 2svp 2svs 2svt 2svv 2svnj 2svdž 2svz 2svš 2svđ 2svć 2svč 2svž 2špb 2špc 2špd 2špf 2špg 2šph 2špk 2špm 2špn 2špp 2šps 2špt 2špv 2špnj 2špdž 2špz 2špš 2špđ 2špć 2špč 2špž 2švb 2švc 2švd 2švf 2švg 2švh 2švk 2švm 2švn 2švp 2švs 2švt 2švv 2švnj 2švdž 2švz 2švš 2švđ 2švć 2švč 2švž 2ždb 2ždc 2ždd 2ždf 2ždg 2ždh 2ždk 2ždm 2ždn 2ždp 2žds 2ždt 2ždnj 2žddž 2ždz 2ždš 2ždđ 2ždć 2ždč 2žgb 2žgc 2žgd 2žgf 2žgg 2žgh 2žgk 2žgm 2žgn 2žgp 2žgs 2žgt 2žgnj 2žgdž 2žgz 2žgš 2žgđ 2žgć 2žgč 2žgž 2zdb 2zdc 2zdd 2zdf 2zdg 2zdh 2zdk 2zdm 2zdn 2zdp 2zds 2zdt 2zdnj 2zddž 2zdz 2zdš 2zdđ 2zdć 2zdč 2zgb 2zgc 2zgd 2zgf 2zgg 2zgh 2zgk 2zgm 2zgn 2zgp 2zgs 2zgt 2zgnj 2zgdž 2zgz 2zgš 2zgđ 2zgć 2zgč 2zgž 2žvb 2žvc 2žvd 2žvf 2žvg 2žvh 2žvk 2žvm 2žvn 2žvp 2žvs 2žvt 2žvv 2žvnj 2žvdž 2žvz 2žvš 2žvđ 2žvć 2žvč 2žvž 2žbb 2žbc 2žbd 2žbf 2žbg 2žbh 2žbk 2žbm 2žbn 2žbp 2žbs 2žbt 2žbv 2žbnj 2žbdž 2žbz 2žbš 2žbđ 2žbć 2žbč 2žbž 2zvb 2zvc 2zvd 2zvf 2zvg 2zvh 2zvk 2zvm 2zvn 2zvp 2zvs 2zvt 2zvv 2zvnj 2zvdž 2zvz 2zvš 2zvđ 2zvć 2zvč 2zvž 2zbb 2zbc 2zbd 2zbf 2zbg 2zbh 2zbk 2zbm 2zbn 2zbp 2zbs 2zbt 2zbv 2zbnj 2zbdž 2zbz 2zbš 2zbđ 2zbć 2zbč 2zbž 2žmb 2žmc 2žmd 2žmf 2žmg 2žmh 2žmk 2žmm 2žmn 2žmp 2žms 2žmt 2žmv 2žmnj 2žmdž 2žmz 2žmš 2žmđ 2žmć 2žmč 2žmž 2smb 2smc 2smd 2smf 2smg 2smh 2smk 2smm 2smn 2smp 2sms 2smt 2smv 2smnj 2smdž 2smz 2smš 2smđ 2smć 2smč 2smž 2zmb 2zmc 2zmd 2zmf 2zmg 2zmh 2zmk 2zmm 2zmn 2zmp 2zms 2zmt 2zmv 2zmnj 2zmdž 2zmz 2zmš 2zmđ 2zmć 2zmč 2zmž 2šmb 2šmc 2šmd 2šmf 2šmg 2šmh 2šmk 2šmm 2šmn 2šmp 2šms 2šmt 2šmv 2šmnj 2šmdž 2šmz 2šmš 2šmđ 2šmć 2šmč 2šmž 2scb 2scc 2scd 2scf 2scg 2sch 2sck 2scl 2scm 2scn 2scp 2sclj 2scr 2scs 2sct 2scnj 2scdž 2scz 2scš 2scđ 2scć 2scč 2scž 2šcb 2šcc 2šcd 2šcf 2šcg 2šch 2šck 2šcl 2šcm 2šcn 2šcp 2šclj 2šcr 2šcs 2šct 2šcnj 2šcdž 2šcz 2šcš 2šcđ 2šcć 2šcč 2šcž 2ščb 2ščc 2ščd 2ščf 2ščg 2ščh 2ščj 2ščk 2ščl 2ščm 2ščn 2ščp 2ščlj 2ščr 2ščs 2ščt 2ščnj 2ščdž 2ščz 2ščš 2ščđ 2ščć 2ščč 2ščž 2hvb 2hvc 2hvd 2hvf 2hvg 2hvh 2hvk 2hvm 2hvn 2hvp 2hvs 2hvt 2hvnj 2hvdž 2hvz 2hvš 2hvđ 2hvć 2hvč 2hvž 2ž3vl 2ž3vlj 2c3vl 2c3vlj 2z3vl 2z3vlj 2š3vl 2š3vlj 2č3vl 2č3vlj 2č3vj 2s3vlj 2d3vl 2d3vlj 2d3vr 2k3vj 2k3vl 2k3vlj 2t3vj 2t3vl 2t3vlj 2g3vj 2g3vl 2g3vlj 2g3vr 2h3vj 2h3vl 2h3vlj 2h3vr 2ž3mj 2ž3ml 2ž3mlj 2ž3mr 2z3ml 2z3mr 2š3mj 2š3ml 2š3mlj 2š3cj 2š3cv 2š3čv 2š3tj 2š3tl 2š3tlj 2s3tl 2s3kj 2s3klj 2š3pj 2š3pl 2š3plj 2ž3dj 2ž3dl 2ž3dlj 2ž3dv 2ž3gj 2ž3gl 2ž3glj 2ž3gr 2ž3gv 2z3dl 2z3dlj 2z3dv 2z3gj 2z3glj 2ž3bj 2ž3bl 2ž3blj 2ž3br 2z3blj .a4e2ro .be4o .bi4o .ge4o .za3g2n .za3t2ka .iza3g2n .iza3t2k .iz3g2n .izd2no .izd2nu .izd2na .izr2k .ist2k .nag2n .nag2nj .na3d2nev .na3d2nič .na3d2nic .na3t2kah .na3t2kam .na3t2kasm .na3t2kast .oda3g2n .oda3d2n .od3g2n .od3m2n .o3t2kah .o3t2kam .o3t2kasm .o3t2kast .po3g2n .po3d2nev .po3m2n .po3m2nj .po3r2v .po3r2đ .po3t2kah .po3t2kam .po3t2kat .po3t2kav .pred3m2n .pred3m2nj .pre3t2kah .pre3t2kam .pre3t2kat .pro3g2n .pro3t2ki .pro3t2ka .raza3g2n .raz3g2n .raz3d2ni .raza3t2ka .u3g2mi .u3g2n .uza3t2ka 3h2teti 3h2tjeti 3h2tel 3h2tev 3h2tenj 3h2tjel 3h2tjev 3h2tjenj 3g2degod. 3g2djegod. 3g2dekak 3g2dekad 3g2djekak 3g2djekad ni3g2de. ne3g2de. ni3g2dje. ne3g2dje. 3b2det 3b2denj 3b2djet 3b2djenj 3g2mil 3g2milj 3g2miz 3g2nos 3g2noz 3g2noj 3g2naj 3g2nezd 3g2nijezd 3g2nežđ 3g2niježđ 3g2nev 3g2njev 3g2njav 3g2njes 3g2njet 3g2nječ 3g2njil 3g2njio 3g2njilj 3g2njit 3g2njur 3k2nez 3k2než 3k2njiž 3k2njig 3m2nož 3m2nog 3m2naž 3p2sik 3p2sič 3p2sov 3p2suj 3r2đa 3s2fer 3t2mast 3t2mul 3t2muo 3t2mulj 3t2mur 3c2miz 3c2mak 3c2mač 3c2mok 3č2lan 3č2lanj 3r2je 4r3jem 4r3je. .be2z3j .be2z3l .be2z3m .be2z3n .be2z3lj .be2z3r .be2z3v .be2z3nj .be2z3b .be2z3d .be2z3g .be2z3i .be2z3o .be2z3u .be2z3alkohol .be2z3atomsk .be3z4bedn .be3z4beda .be3z4bjedn .be3z4bjeda .be3z4beli .be3z4bol .be3z4vučn .be3z4vuča .be3z4istan .be3z4isten .be3z4jak .be3z4jač .be3z4lobn .be3z4loba .be3z4načaj .be3z4račn .be3z4rača .be3z4up .be3z4ub .be2s3c .be2s3k .be2s3p .be2s3t .be3s4krupul .be3s4pokojn .be3s4pokoja .be3s4porn .be3s4pora .be3s4tvar .be3s4tid .be3s4tija .be3s4tilu .be3s4tilj .be3s4trana .be3s4tras .bes4tseler .be2š3ć .be2š3č .va2n3evr .va2n3ustav .i2z3b .i2z3d .i2z3g .i2z3j .i2z3l .i2z3m .i2z3n .i2z3lj .i2z3r .i2z3v .i2z3nj .i2z3i .i2z3o .i2z3u .i2z3bija .i2z3biva .i2z3vedi .i2z3vedn .i2z3vedb .i2z3vede .i2z3daj .i2z3aba .i2z3aka .i2z3anal .i3z4bav .i3z4bičkava .i3z4bleušan .i3z4bojak .i3z4bojk .i3z4vali .i3z4valu .i3z4vala .i3z4vale .i3z4valji .i3z4vižd .i3z4viiskr .i3z4vija .i3z4vijen .i3z4vin .i3z4vir .i3z4vinj .i3z4vitop .i3z4vjed .i3z4vojac .i3z4vojc .i3z4vor .i3z4gomet .i3z4gred .i3z4grn .i3z4grt .i3z4drav .i3z4iđ .i3z4id .i3z4imi .i3z4ježlj .i3z4loz .i3z4lož .i3z4log .i3z4lopać .i3z4nim .i3z4noj .iz4oanem .iz4oanom .iz4obat .iz4obron .iz4ogam .iz4ogeo .iz4oglos .iz4ogon .iz4ograf .iz4odim .iz4odin .iz4odoz .iz4oklin .iz4okolon .i3z4olat .i3z4olac .i3z4olir .i3z4olov .iz4oleks .iz4oluks .iz4omer .iz4ometr .iz4omorf .iz4onef .iz4onom .iz4opat .iz4oper .iz4ople .iz4opol .iz4opsef .iz4orah .iz4osei .iz4osint .iz4osist .iz4oskel .iz4oskop .iz4ostaz .iz4oste .iz4otah .iz4otal .iz4oter .iz4oton .iz4otop .iz4otro .iz4ofon .iz4ofot .iz4ohal .iz4ohaz .iz4ohel .iz4ohij .iz4ohim .iz4ohit .iz4ohips .iz4ohor .iz4ohro .i3z4rael .i3z4railj .i3z4rači .i3z4un .i3z4upč .i2s3c .i2s3k .i2s3p .i2s3t .i3s4kak .i3s4kat .i3s4kanj .i3s4kariot .i3s4kvas .i3s4kvrč .i3s4kin .i3s4kita .i3s4konsk .i3s4koč .i3s4kram .i3s4krit .i3s4kriš .i3s4krič .i3s4kric .i3s4krat .i3s4kren .i3s4krenj .i3s4kroj .i3s4krsn .i3s4krsa .i3s4kuplja .i3s4lam .i3s4lab .i3s4leđ .i3s4led .i3s4lijeđ .i3s4lijed .i3s4ljeđ .i3s4ljed .i3s4lik .i3s4lin .i3s4lov .i3s4luš .i3s4luž .i3s4me .i3s4mije .i3s4mje .i3s4pav .i3s4paljiv .i3s4pira .i3s4plit .i3s4plić .i3s4pokoj .i3s4polin .i3s4pon .i3s4porav .i3s4pravi .i3s4pravk .i3s4pravn .i3s4pravlj .i3s4prava .i3s4pupč .i3s4pur .i3s4red .i3s4rk .i3s4tavi .i3s4tavlj .i3s4takn .i3s4tam .i3s4tar .i3s4tas .i3s4tać .i3s4tin .i3s4tir .i3s4tic .i3s4tifan .i3s4tok .i3s4tori .i3s4točn .i3s4točnj .i3s4toča .i3s4trav .i3s4trad .i3s4tran .i3s4trić .i3s4triž .i3s4tric .i3s4trug .i3s4tup .i3s4uk .i3s4us .i3s4ut .i3s4uš .i2ž3đ .i2š3ć .i2š3č .iz3be2z3obr .iz3be2z3um .iz3va2n3evr .na2d3l .na2d3lj .na2d3v .na3d4val .na3d4vesi .na3d4vest .na3d4vij .na3d4vit .na3d4vla .na3d4voje .na3d4vor .na2d3igr .na2d3inž .na2d3ina .na2d3isk .na2d3jah .na2d3jač .na2d3jek .na2d3jez .na2d3ječ .na2d3jun .na3d4lan .na3d4leš .na3d4lež .na2d3ora .na2d3oso .na2d3ose .na2d3osje .na2d3ofi .na2d3oč .na2d3ran .na2d3rač .na2d3rast .na2d3rašć .na2d3real .na2d3rep .na2d3ruk .na2d3ruč .na2d3rug .na2d3udar .na2d3um .na2d3uč .na2j3a .na2j3e .na2j3i .na2j3o .na2j3u .na3j4avi .na3j4avlj .na3j4ava .na3j4ave .na3j4adi .na3j4ada .na3j4ade .na3j4aži .na3j4azi .na3j4ako .na3j4aka .na3j4alo .na3j4ami .na3j4aml .na3j4amn .na3j4ari .na3j4arm .na3j4arc .na3j4ati .na3j4auk .na3j4ah .na3j4aš .na3j4edi .na3j4edn .na3j4edr .na3j4eda .na3j4eži .na3j4ežu .na3j4eže .na3j4ezn .na3j4ezd .na3j4esti .na3j4etk .na3j4ec .na3j4uri .na3j4uren .o2b3j .o2b3lj .o2b3r .obe2z3b .obe2z3d .obe2z3g .obe2z3j .obe2z3l .obe2z3m .obe2z3n .obe2z3o .obe2z3lj .obe2z3r .obe2z3u .obe2z3v .obe3z4vij .obe3z4nan .obe3z4nanj .obe3z4nač .obe3z4ub .obe2s3c .obe2s3k .obe2s3p .obe2s3t .obe3s4tan .obe3s4tij .obe3s4tran .obe2š3ć .obe2š3č .o2b3igr .o2b3istin .o2b3istinj .o3b4jek .o3b4jer .o3b4jesi .o3b4jet .o3b4ješ .o2b3laj .o2b3lam .o2b3lakš .o2b3lask .o2b3lep .o2b3let .o2b3leć .o2b3lež .o2b3leg .o2b3lijep .o2b3lijet .o2b3lijež .o2b3lijeg .o2b3leden .o2b3liv .o2b3lizat .o2b3lizav .o2b3lizi .o2b3list .o2b3loka .o2b3luk .o2b3luč .o3b4ljan .o3b4ljut .o3b4ljuz .o2b3oru .o3b4raže .o3b4razi .o3b4razn .o3b4razo .o3b4razu .o3b4raza .o3b4razd .o3b4ramb .o3b4ran .o3b4ranj .o3b4rat .o3b4rać .o3b4rašn .o3b4rašč .o3b4rv .o3b4rđ .o3b4rem .o3b4res .o3b4ređ .o3b4reč .o3b4rež .o3b4rec .o3b4red .o3b4reti .o3b4retn .o3b4rij .o3b4ris .o3b4rit .o3b4riv .o3b4rič .o3b4ric .o3b4rk .o3b4rl .o3b4rn .o3b4rlj .o3b4rs .o3b4rt .o3b4rš .o3b4rč .o3b4rok .o3b4ron .o3b4ronj .o3b4roć .o3b4roč .o3b4rova .o3b4rovc .o3b4ruk .o3b4run .o3b4rus .o3b4runj .o3b4ruš .o3b4ruč .o2b3ubož .o2b3uz .o2b3už .o2b3ud .o2b3umi .o2b3umj .o2b3umr .o2b3ume .o2d3j .o2d3l .o2d3lj .o2d3r .o2d3v .o2d3arg .o3d4vaj .o3d4važ .o3d4vesn .o3d4vest .o3d4vesa .o3d4vikav .o3d4vikn .o3d4vis .o3d4vić .o3d4voj .o2d3igr .o2d3i2z3v .o2d3i2z3d .o2d3isk .o2d3isti .o3d4jel .o3d4jen .o3d4jev .o3d4jeć .o3d4laz .o3d4laž .o3d4lag .o3d4laka .o3d4luk .o3d4luč .o2d3ozd .o2d3ozg .o2d3ok .o2d3onl .o2d3ono .o2d3onu .o2d3ond .o3d4rani .o3d4rano .o3d4ranu .o3d4rana .o3d4rane .o3d4raz .o3d4rać .o3d4raž .o3d4rapi .o3d4raplj .o3d4rapa .o3d4rači .o3d4rven .o3d4rvenj .o3d4rveč .o3d4rem .o3d4ren .o3d4ret .o3d4ređ .o3d4red .o3d4rl .o3d4rn .o3d4rp .o3d4rlj .o3d4rt .o3d4rž .o3d4rin .o3d4rinj .o3d4riš .o3d4rič .o3d4rib .o3d4ric .o3d4ron .o3d4ronj .o3d4ruž .o3d4rug .o2d3uvi .o2d3uve .o2d3uzi .o2d3uzl .o2d3uzd .o2d3uze .o2d3uk .o2d3ul .o2d3um .o2d3uč .po2d3adm .po2d3varij .po2d3vez .po2d3več .po2d3vež .po2d3vik .po2d3vil .po2d3vir .po2d3vinj .po2d3vlas .po2d3vlaš .po2d3voz .po2d3vođ .po2d3vož .po2d3vod .po2d3vrat .po2d3vrać .po2d3vrć .po2d3vrž .po2d3vrg .po2d3vris .po2d3vrs .po2d3vuć .po2d3igr .po2d3izv .po2d3j .po3d4jen .po3d4ječ .po2d3lakat .po2d3lakt .po2d3lep .po2d3let .po2d3leć .po2d3lež .po2d3leg .po2d3liz .po2d3lijep .po2d3lijet .po2d3lijeć .po2d3lijež .po2d3lijeg .po2d3list .po2d3lok .po2d3lom .po2d3lup .po2d3luč .po2d3luž .po2d3ljut .po2d3okn .po2d3oš .po2d3oč .po2d3of .po2d3ravn .po2d3ravnj .po2d3rad .po2d3ra2z3d .po2d3razr .po2d3razu .po2d3ram .po2d3ran .po2d3ras .po2d3ranj .po2d3rep .po2d3res .po2d3rez .po2d3rik .po2d3rit .po2d3ron .po2d3rov .po2d3rož .po2d3ruk .po2d3rub .po2d3ruči .po2d3ručn .po2d3ruča .po2d3upla .po2d3usm .po2d3usn .pre2d3j .pre2d3v .pre3d4vaj .pre3d4var .pre3d4vest .pre3d4voji .pre3d4voja .pre3d4voje .pre3d4vor .pre3d4vos .pre3d4jen .pre2d3igr .pre2d3id .pre2d3izb .pre2d3ispi .pre2d3isto .pre2d3istr .pre2d3obj .pre2d3odre .pre2d3okus .pre2d3osv .pre2d3ose .pre2d3osje .pre2d3rat .pre2d3rač .pre2d3rad .pre2d3ruč .pre2d3ubeđ .pre2d3ubijeđ .pre2d3ubjeđ .pre2d3uver .pre2d3uvjer .pre2d3uvjet .pre2d3ugov .pre2d3udar .pre2d3upis .pre2d3uslo .proti2v3akc .proti2v3otr .proti2v3of .proti2v3r .proti2v3us .proti2v3ud .ra2ž3đ .ra2z3b .ra2z3d .ra2z3e .ra2z3g .ra2z3i .ra2z3j .ra2z3l .ra2z3m .ra2z3n .ra2z3lj .ra2z3r .ra2z3v .ra2z3nj .ra2z3anal .ra3z4ban .ra3z4bar .ra3z4bau .ra3z4bad .ra3z4bašur .ra3z4boj .ra3z4bor .ra3z4val .ra3z4vede .ra3z4vest .ra3z4vigo .ra3z4viju .ra3z4vija .ra3z4vije .ra3z4vit .ra3z4vić .ra3z4voj .ra3z4von .ra3z4vrat .ra3z4vrać .ra3z4vrt .ra3z4vrć .ra3z4gađ .ra3z4grt .ra3z4ev .ra3z4ij .ra3z4il .ra3z4in .ra3z4ir .ra3z4it .ra3z4iz .ra3z4iđ .ra3z4ić .ra3z4id .ra3z4laz .ra3z4lag .ra3z4lik .ra3z4lič .ra3z4loz .ra3z4lož .ra3z4log .ra3z4met .ra3z4meć .ra3z4mrsk .ra3z4nat .ra2z3obl .ra2z3obr .ra2z3oba .ra2z3od .ra2z3orat .ra2z3orav .ra2z3ort .ra2z3oru .ra2z3ot .ra3z4red .ra3z4rok .ra3z4roč .ra2z3uve .ra2z3udi .ra2z3uda .ra2z3udb .ra2z3uzi .ra2z3uzd .ra2z3uze .ra2z3ular .ra2z3umr .ra2s3c .ra2s3k .ra2s3p .ra2s3t .ra3s4kak .ra3s4kand .ra3s4kin .ra3s4klap .ra3s4klanj .ra3s4klad .ra3s4klon .ra3s4klopi .ra3s4kloplj .ra3s4klopa .ra3s4koš .ra3s4krop .ra3s4paj .ra3s4pav .ra3s4peti .ra3s4peto .ra3s4peta .ra3s4pete .ra3s4piku .ra3s4pinj .ra3s4plin .ra3s4plinj .ra3s4pn .ra3s4polož .ra3s4pon .ra3s4por .ra3s4prav .ra3s4prem .ra3s4rđ .ra3s4rd .ra3s4re .ra3s4taj .ra3s4tan .ra3s4tat .ra3s4tav .ra3s4tenj .ra3s4til .ra3s4tir .ra3s4tis .ra3s4tit .ra3s4tinj .ra3s4toj .ra3s4trel .ra3s4tret .ra3s4troj .ra3s4trt .ra3s4tup .ra3s4tur .ra3s4tuć .ra4s5turč .ra2š3ć .ra2š3č .ra3š4ćenj .ra3š4čić .u2z3b .u2z3d .u2z3g .u2z3i .u2z3j .u2z3l .u2z3m .u2z3n .u2z3lj .u2z3r .u2z3v .u2z3nj .u3z4bor .u3z4van .u3z4vat .u3z4viž .u3z4vijo .u3z4viju .u3z4vija .u3z4vije .u3z4voj .u3z4dic .u2z3igr .u2z3inat .u2z3iskr .u3z4lan .u3z4lat .u3z4lim .u3z4lit .u3z4lić .u3z4lic .u3z4lov .u3z4ludob .u3z4nak .u3z4nač .u3z4neve .u3z4nevje .u3z4nič .u3z4nic .u3z4noj .u2z3obest .u2z3obijest .u2z3orat .u2z3orav .u2z3oho .u3z4ret .u3z4rev .u3z4rijet .u3z4rijev .u3z4rn .u3z4rnj .u3z4roko .u3z4roku .u3z4roka .u3z4roč .u3z4ruj .u2z3ugar .u2s3c .u2s3k .u2s3p .u3s4kak .u3s4klađ .u3s4klad .u3s4ko .u4s5kom .u4s5kov .u4s5koš .u4s5koko .u4s5kolu .u4s5kole .u4s5kopa .u4s5kora .u4s5kosi .u4s5kotr .u3s4kup .u3s4pav .u3s4palo .u3s4peh .u3s4pel .u3s4pem .u3s4pet .u3s4pev .u3s4peš .u3s4pjeh .u3s4pjel .u3s4pjem .u3s4pjet .u3s4pjev .u3s4pješ .u3s4pent .u3s4pija .u3s4pije .u3s4pijuš .u3s4pikuš .u3s4pon .u3s4pori .u3s4pora .u3s4poren .u3s4porenj .u3s4poreč .u3s4posob .u3s4premi .u3s4prema .u3s4rk .u3s4rn .u3s4rp .u3s4rlj .u3s4rt .u3s4rđ .u3s4rž .u3s4ra .u3s4rd .u3s4re .u3s4rijed .u2s3talas .u2s3tara .u2s3tvrđ .u2s3tvrd .u2s3ter .u2s3teć .u2s3teg .u2s3tov .u2s3traj .u2s3tral .u2s3trg .u2s3trep .u2s3tres .u2s3treb .u2s3trk .u2s3trn .u2s3trp .u2s3trć .u2s3trč .u2s3tum .u2s3tur .u2s3tuć .u2š3ć .u2š3č .a2b3alij .a2b3anac .a2b3evak .a2b3erac .a2b3erir .a2b3irit .a2b3ju .a2b3la .a2b3leg .a2b3lep .a2b3lok .a2b3lu .a2b3orig .a2b3reak .a2b3rog .a2b3uzus .a2d3erac .a2d3verb .a2d3j .a2d3lat .a2d3ren .a2d3rog .a3g2nos .a3g2noz .a2nabap .a2nabaz .a2nabat .a2nabio .a2nabol .a2nagen .a2nagno .a2n3ago .a2nagra .a2nadem .a2nadipl .a2nadoz .a2n3a4e2ro .a2nakal .a2nakam .a2nakat .a2nakef .a2nakla .a2nakli .a2nakoj .a2n3akuz .a2n3alg .a2n3ald .a2nalep .a2naliz .a2nalis .a2nalit .a2n3amert .a2namne .a2n3andr .a2naneo .a2n3ant .a2napla .a2naple .a2napne .a2napno .a2napro .a2napti .a2n3apto .a2nart .a2n3arh .a2nasar .a2nasei .a2naspaz .a2nasta .a2nastig .a2nastom .a2natim .a2natom .a2natoc .a2natre .a2natri .a2natro .a2nafaz .a2n3afij .a2nafila .a2nafon .a2n3afrod .a2nakol .a2nakron .a2nakru .a2n3alfa .a2nafor .a2nahor .a2nahro .a2n3eger .a2n3ekl .a2n3ekum .a2n3elek .a2n3ener .a2n3epi .a2neor .a2n3erg .a2n3erit .a2n3este .a2n3idr .a2n3izog .a2n3izom .a2n3izur .a2n3irid .a2n3ovar .a2n3oks .a2n3opis .a2n3orh .a2n3oft .a2n3org .di2s3akor .di2s3junk .di2s3kval .di2s3kont .di2s3kord .di2s3kre .di2s3kri .di2s3kur .di2s3lo .di2s3orij .di2s3parit .di2s3poz .di2s3pon .di2s3prop .di2s3ton .di2s3trak .i2n3abrup .i2n3adek .i2n3akur .i2n3akce .i2n3amor .i2n3anic .i2n3aplik .i2n3apst .i2n3art .i2n3augur .i2n3aura .i2n3afek .i2n3evid .i2n3eg .i2n3ed .i2n3ekv .i2n3eks .i2n3elig .i2n3epc .i2n3efek .i2n3obl .i2nogen .i2nokor .i2n3okup .i2n3oper .i2n3opor .i2n3opse .i2n3ofic .i2n3umbr .i2n3unda .i2n3unk .i2n3util .inte2r3i .inte2r3o .inte2r3u .inte2r3a .inte2r3e .inte3r4egn .inte3r4esi .inte3r4esn .inte3r4eso .inte3r4esu .inte3r4esa .inte3r4ese .inte3r4eždž .inte3r4ije .inte2r3je .inte3r4ogat .juri2s3k .juri2s3p .nu2z3bel .nu2z3bilj .nu2z3ljub .nu2z3re .nu2z3rje .nu2z3už .nu2s3pos .nu2s3pro .po2st3egz .po2st3indu .po2st3lim .po2st3onk .po2st3oper .su2b3a .su2b3l .su3b4aš .su2b3inv .su2b3junk .su2b3oks .su2b3rep .su2b3rog .su2b3ord .supe2r3i .supe2r3o .supe2r3u .supe2r3a .supe2r3e .supe3r4ior .tran2s3a .tran2s3c .tran2s3e .tran2s3k .tran2s3l .tran2s3m .tran2s3n .tran2s3o .tran2s3p .tran2s3t .tran2s3u .tran2s3v .tran2s3nj .tran3s4ep .tran3s4kri .tran3s4um .tran3s4ud .a2n3jon .i2n3jek .i2n3jur .i2n3just .o2d3žal .o2d3žalj .o2d3ži .o2d3žv .o2d3že .pre2d3ži .pre2d3že .na2d3ždr .na2d3žnj .na2d3že .na2d3žanj .na2d3žir .na2d3živ .na2d3žup l2j n2j d2ž 2dž.",
  ["lefthyphenmin"]=1,
  ["length"]=50368,
  ["n"]=5321,
  ["righthyphenmin"]=1,
 },
 ["version"]="1.001",
}