return {
 ["aleph"]={
  "Alephminorversion",
  "Alephrevision",
  "Alephversion",
 },
 ["etex"]={
  "botmarks",
  "clubpenalties",
  "currentgrouplevel",
  "currentgrouptype",
  "currentifbranch",
  "currentiflevel",
  "currentiftype",
  "currentstacksize",
  "detokenize",
  "dimexpr",
  "displaywidowpenalties",
  "everyeof",
  "firstmarks",
  "fontchardp",
  "fontcharht",
  "fontcharic",
  "fontcharwd",
  "glueexpr",
  "glueshrink",
  "glueshrinkorder",
  "gluestretch",
  "gluestretchorder",
  "gluetomu",
  "ifcsname",
  "ifdefined",
  "iffontchar",
  "interactionmode",
  "interlinepenalties",
  "lastlinefit",
  "lastnodetype",
  "marks",
  "muexpr",
  "mutoglue",
  "numexpr",
  "pagediscards",
  "parshapedimen",
  "parshapeindent",
  "parshapelength",
  "predisplaydirection",
  "protected",
  "savinghyphcodes",
  "savingvdiscards",
  "scantokens",
  "showgroups",
  "showifs",
  "showtokens",
  "splitbotmarks",
  "splitdiscards",
  "splitfirstmarks",
  "topmarks",
  "tracingassigns",
  "tracinggroups",
  "tracingifs",
  "tracingnesting",
  "unexpanded",
  "unless",
  "widowpenalties",
 },
 ["luatex"]={
  "Uabove",
  "Uabovewithdelims",
  "Uatop",
  "Uatopwithdelims",
  "Udelcode",
  "Udelimited",
  "Udelimiter",
  "Udelimiterover",
  "Udelimiterunder",
  "Uhextensible",
  "Uleft",
  "Umathaccent",
  "Umathaccentbasedepth",
  "Umathaccentbaseheight",
  "Umathaccentbottomovershoot",
  "Umathaccentbottomshiftdown",
  "Umathaccentextendmargin",
  "Umathaccentsuperscriptdrop",
  "Umathaccentsuperscriptpercent",
  "Umathaccenttopovershoot",
  "Umathaccenttopshiftup",
  "Umathaccentvariant",
  "Umathadapttoleft",
  "Umathadapttoright",
  "Umathaxis",
  "Umathbottomaccentvariant",
  "Umathchar",
  "Umathchardef",
  "Umathcode",
  "Umathconnectoroverlapmin",
  "Umathdegreevariant",
  "Umathdelimiterextendmargin",
  "Umathdelimiterovervariant",
  "Umathdelimiterpercent",
  "Umathdelimitershortfall",
  "Umathdelimiterundervariant",
  "Umathdenominatorvariant",
  "Umathdictdef",
  "Umathexheight",
  "Umathextrasubpreshift",
  "Umathextrasubprespace",
  "Umathextrasubshift",
  "Umathextrasubspace",
  "Umathextrasuppreshift",
  "Umathextrasupprespace",
  "Umathextrasupshift",
  "Umathextrasupspace",
  "Umathflattenedaccentbasedepth",
  "Umathflattenedaccentbaseheight",
  "Umathflattenedaccentbottomshiftdown",
  "Umathflattenedaccenttopshiftup",
  "Umathfractiondelsize",
  "Umathfractiondenomdown",
  "Umathfractiondenomvgap",
  "Umathfractionnumup",
  "Umathfractionnumvgap",
  "Umathfractionrule",
  "Umathfractionvariant",
  "Umathhextensiblevariant",
  "Umathlimitabovebgap",
  "Umathlimitabovekern",
  "Umathlimitabovevgap",
  "Umathlimitbelowbgap",
  "Umathlimitbelowkern",
  "Umathlimitbelowvgap",
  "Umathlimits",
  "Umathnoaxis",
  "Umathnolimits",
  "Umathnolimitsubfactor",
  "Umathnolimitsupfactor",
  "Umathnumeratorvariant",
  "Umathopenupdepth",
  "Umathopenupheight",
  "Umathoperatorsize",
  "Umathoverbarkern",
  "Umathoverbarrule",
  "Umathoverbarvgap",
  "Umathoverdelimiterbgap",
  "Umathoverdelimitervariant",
  "Umathoverdelimitervgap",
  "Umathoverlayaccentvariant",
  "Umathoverlinevariant",
  "Umathphantom",
  "Umathprimeraise",
  "Umathprimeraisecomposed",
  "Umathprimeshiftdrop",
  "Umathprimeshiftup",
  "Umathprimespaceafter",
  "Umathprimevariant",
  "Umathquad",
  "Umathradicaldegreeafter",
  "Umathradicaldegreebefore",
  "Umathradicaldegreeraise",
  "Umathradicalextensibleafter",
  "Umathradicalextensiblebefore",
  "Umathradicalkern",
  "Umathradicalrule",
  "Umathradicalvariant",
  "Umathradicalvgap",
  "Umathruledepth",
  "Umathruleheight",
  "Umathskeweddelimitertolerance",
  "Umathskewedfractionhgap",
  "Umathskewedfractionvgap",
  "Umathsource",
  "Umathspaceafterscript",
  "Umathspacebeforescript",
  "Umathspacebetweenscript",
  "Umathstackdenomdown",
  "Umathstacknumup",
  "Umathstackvariant",
  "Umathstackvgap",
  "Umathsubscriptsnap",
  "Umathsubscriptvariant",
  "Umathsubshiftdown",
  "Umathsubshiftdrop",
  "Umathsubsupshiftdown",
  "Umathsubsupvgap",
  "Umathsubtopmax",
  "Umathsupbottommin",
  "Umathsuperscriptsnap",
  "Umathsuperscriptvariant",
  "Umathsupshiftdrop",
  "Umathsupshiftup",
  "Umathsupsubbottommax",
  "Umathtopaccentvariant",
  "Umathunderbarkern",
  "Umathunderbarrule",
  "Umathunderbarvgap",
  "Umathunderdelimiterbgap",
  "Umathunderdelimitervariant",
  "Umathunderdelimitervgap",
  "Umathunderlinevariant",
  "Umathuseaxis",
  "Umathvextensiblevariant",
  "Umathvoid",
  "Umathxscale",
  "Umathyscale",
  "Umiddle",
  "Uoperator",
  "Uover",
  "Uoverdelimiter",
  "Uoverwithdelims",
  "Uradical",
  "Uright",
  "Uroot",
  "Urooted",
  "Uskewed",
  "Uskewedwithdelims",
  "Ustartdisplaymath",
  "Ustartmath",
  "Ustartmathmode",
  "Ustopdisplaymath",
  "Ustopmath",
  "Ustopmathmode",
  "Ustretched",
  "Ustretchedwithdelims",
  "Uunderdelimiter",
  "Uvextensible",
  "additionalpageskip",
  "adjacentdemerits",
  "adjustspacing",
  "adjustspacingshrink",
  "adjustspacingstep",
  "adjustspacingstretch",
  "advanceby",
  "afterassigned",
  "aftergrouped",
  "aliased",
  "aligncontent",
  "alignmark",
  "alignmentcellsource",
  "alignmentwrapsource",
  "aligntab",
  "allcrampedstyles",
  "alldisplaystyles",
  "allmainstyles",
  "allmathstyles",
  "allscriptscriptstyles",
  "allscriptstyles",
  "allsplitstyles",
  "alltextstyles",
  "alluncrampedstyles",
  "allunsplitstyles",
  "amcode",
  "associateunit",
  "atendoffile",
  "atendoffiled",
  "atendofgroup",
  "atendofgrouped",
  "attribute",
  "attributedef",
  "automaticdiscretionary",
  "automatichyphenpenalty",
  "automigrationmode",
  "autoparagraphmode",
  "balanceadjdemerits",
  "balancebottomskip",
  "balanceboundary",
  "balancebreakpasses",
  "balancechecks",
  "balanceemergencyshrink",
  "balanceemergencystretch",
  "balancefinalpenalties",
  "balancelineheight",
  "balancelooseness",
  "balancepasses",
  "balancepenalty",
  "balanceshape",
  "balanceshapebottomspace",
  "balanceshapetopspace",
  "balanceshapevsize",
  "balancetolerance",
  "balancetopskip",
  "balancevsize",
  "begincsname",
  "beginlocalcontrol",
  "beginmathgroup",
  "beginmvl",
  "beginsimplegroup",
  "bottomskip",
  "boundary",
  "boxadapt",
  "boxanchor",
  "boxanchors",
  "boxattribute",
  "boxdirection",
  "boxfinalize",
  "boxfreeze",
  "boxgeometry",
  "boxinserts",
  "boxlimit",
  "boxlimitate",
  "boxlimitmode",
  "boxmigrate",
  "boxorientation",
  "boxrepack",
  "boxshift",
  "boxshrink",
  "boxsource",
  "boxstretch",
  "boxsubtype",
  "boxtarget",
  "boxtotal",
  "boxvadjust",
  "boxxmove",
  "boxxoffset",
  "boxymove",
  "boxyoffset",
  "brokenpenalties",
  "catcodetable",
  "cccode",
  "cdef",
  "cdefcsname",
  "cfcode",
  "clearmarks",
  "constant",
  "constrained",
  "copymathatomrule",
  "copymathparent",
  "copymathspacing",
  "copysplitdiscards",
  "crampeddisplaystyle",
  "crampedscriptscriptstyle",
  "crampedscriptstyle",
  "crampedtextstyle",
  "csactive",
  "csnamestring",
  "csstring",
  "currentloopiterator",
  "currentloopnesting",
  "currentlysetmathstyle",
  "currentmarks",
  "dbox",
  "defcsname",
  "deferred",
  "detokened",
  "detokenized",
  "dimensiondef",
  "dimexperimental",
  "dimexpression",
  "directlua",
  "discretionaryoptions",
  "divideby",
  "doublepenaltymode",
  "dpack",
  "dsplit",
  "edefcsname",
  "edivide",
  "edivideby",
  "efcode",
  "emergencyleftskip",
  "emergencyrightskip",
  "emptyparagraphmode",
  "endlocalcontrol",
  "endmathgroup",
  "endmvl",
  "endsimplegroup",
  "enforced",
  "etexexprmode",
  "etoks",
  "etoksapp",
  "etokspre",
  "eufactor",
  "everybeforepar",
  "everymathatom",
  "everytab",
  "exapostrophechar",
  "exceptionpenalty",
  "expand",
  "expandactive",
  "expandafterpars",
  "expandafterspaces",
  "expandcstoken",
  "expanded",
  "expandedafter",
  "expandeddetokenize",
  "expandedendless",
  "expandedloop",
  "expandedrepeat",
  "expandparameter",
  "expandtoken",
  "expandtoks",
  "explicitdiscretionary",
  "explicithyphenpenalty",
  "explicititaliccorrection",
  "explicitspace",
  "firstvalidlanguage",
  "fitnessclasses",
  "float",
  "floatdef",
  "floatexpr",
  "flushmarks",
  "flushmvl",
  "fontcharba",
  "fontcharta",
  "fontid",
  "fontidentifier",
  "fontmathcontrol",
  "fontspecdef",
  "fontspecid",
  "fontspecifiedname",
  "fontspecifiedsize",
  "fontspecscale",
  "fontspecslant",
  "fontspecweight",
  "fontspecxscale",
  "fontspecyscale",
  "fonttextcontrol",
  "forcedleftcorrection",
  "forcedrightcorrection",
  "formatname",
  "frozen",
  "futurecsname",
  "futuredef",
  "futureexpand",
  "futureexpandis",
  "futureexpandisap",
  "gdefcsname",
  "givenmathstyle",
  "gleaders",
  "glet",
  "gletcsname",
  "glettonothing",
  "gluespecdef",
  "glyph",
  "glyphdatafield",
  "glyphoptions",
  "glyphscale",
  "glyphscriptfield",
  "glyphscriptscale",
  "glyphscriptscriptscale",
  "glyphslant",
  "glyphstatefield",
  "glyphtextscale",
  "glyphweight",
  "glyphxoffset",
  "glyphxscale",
  "glyphxscaled",
  "glyphyoffset",
  "glyphyscale",
  "glyphyscaled",
  "gtoksapp",
  "gtokspre",
  "hbadnessmode",
  "hccode",
  "hjcode",
  "hmcode",
  "holdingmigrations",
  "hpack",
  "hpenalty",
  "hyphenationmin",
  "hyphenationmode",
  "ifabsdim",
  "ifabsfloat",
  "ifabsnum",
  "ifarguments",
  "ifboolean",
  "ifchkdim",
  "ifchkdimension",
  "ifchkdimexpr",
  "ifchknum",
  "ifchknumber",
  "ifchknumexpr",
  "ifcmpdim",
  "ifcmpnum",
  "ifcondition",
  "ifcramped",
  "ifcstok",
  "ifdimexpression",
  "ifdimval",
  "ifempty",
  "ifflags",
  "iffloat",
  "ifhaschar",
  "ifhastok",
  "ifhastoks",
  "ifhasxtoks",
  "ifinalignment",
  "ifincsname",
  "ifinsert",
  "ifintervaldim",
  "ifintervalfloat",
  "ifintervalnum",
  "iflastnamedcs",
  "iflist",
  "ifmathparameter",
  "ifmathstyle",
  "ifnumexpression",
  "ifnumval",
  "ifparameter",
  "ifparameters",
  "ifrelax",
  "iftok",
  "ifzerodim",
  "ifzerofloat",
  "ifzeronum",
  "ignorearguments",
  "ignoredepthcriterion",
  "ignorenestedupto",
  "ignorepars",
  "ignorerest",
  "ignoreupto",
  "immediate",
  "immutable",
  "indexedsubprescript",
  "indexedsubscript",
  "indexedsuperprescript",
  "indexedsuperscript",
  "indexofcharacter",
  "indexofregister",
  "inherited",
  "initcatcodetable",
  "initialpageskip",
  "initialtopskip",
  "insertbox",
  "insertcopy",
  "insertdepth",
  "insertdistance",
  "insertheight",
  "insertheights",
  "insertlimit",
  "insertlinedepth",
  "insertlineheight",
  "insertmaxdepth",
  "insertmode",
  "insertmultiplier",
  "insertpenalty",
  "insertprogress",
  "insertshrink",
  "insertstorage",
  "insertstoring",
  "insertstretch",
  "insertunbox",
  "insertuncopy",
  "insertwidth",
  "instance",
  "integerdef",
  "lastarguments",
  "lastatomclass",
  "lastboundary",
  "lastchkdimension",
  "lastchknumber",
  "lastleftclass",
  "lastloopiterator",
  "lastnamedcs",
  "lastnodesubtype",
  "lastpageextra",
  "lastparcontext",
  "lastpartrigger",
  "lastrightclass",
  "leftmarginkern",
  "lefttwindemerits",
  "letcharcode",
  "letcsname",
  "letfrozen",
  "letmathatomrule",
  "letmathparent",
  "letmathspacing",
  "letprotected",
  "lettolastnamedcs",
  "lettonothing",
  "linebreakchecks",
  "linebreakoptional",
  "linebreakpasses",
  "linedirection",
  "localbreakpar",
  "localbrokenpenalty",
  "localcontrol",
  "localcontrolled",
  "localcontrolledendless",
  "localcontrolledloop",
  "localcontrolledrepeat",
  "localinterlinepenalty",
  "localleftbox",
  "localleftboxbox",
  "localmiddlebox",
  "localmiddleboxbox",
  "localpretolerance",
  "localrightbox",
  "localrightboxbox",
  "localtolerance",
  "lpcode",
  "luaboundary",
  "luabytecode",
  "luabytecodecall",
  "luacopyinputnodes",
  "luadef",
  "luaescapestring",
  "luafunction",
  "luafunctioncall",
  "luametatexmajorversion",
  "luametatexminorversion",
  "luametatexrelease",
  "luatexbanner",
  "luatexrevision",
  "luatexversion",
  "mathatom",
  "mathatomglue",
  "mathatomskip",
  "mathbackwardpenalties",
  "mathbeginclass",
  "mathboundary",
  "mathcharclass",
  "mathcharfam",
  "mathcharslot",
  "mathcheckfencesmode",
  "mathclass",
  "mathdictgroup",
  "mathdictionary",
  "mathdictproperties",
  "mathdirection",
  "mathdiscretionary",
  "mathdisplaymode",
  "mathdisplaypenaltyfactor",
  "mathdisplayskipmode",
  "mathdoublescriptmode",
  "mathendclass",
  "matheqnogapstep",
  "mathfontcontrol",
  "mathforwardpenalties",
  "mathgluemode",
  "mathgroupingmode",
  "mathinlinepenaltyfactor",
  "mathleftclass",
  "mathlimitsmode",
  "mathmainstyle",
  "mathnolimitsmode",
  "mathparentstyle",
  "mathpenaltiesmode",
  "mathpretolerance",
  "mathrightclass",
  "mathrulesfam",
  "mathrulesmode",
  "mathscale",
  "mathscriptsmode",
  "mathslackmode",
  "mathspacingmode",
  "mathstack",
  "mathstackstyle",
  "mathstyle",
  "mathstylefontid",
  "mathsurroundmode",
  "mathsurroundskip",
  "maththreshold",
  "mathtolerance",
  "meaningasis",
  "meaningful",
  "meaningfull",
  "meaningles",
  "meaningless",
  "mugluespecdef",
  "multiplyby",
  "mutable",
  "mvlcurrentlyactive",
  "nestedloopiterator",
  "noaligned",
  "noatomruling",
  "noboundary",
  "nohrule",
  "nomathchar",
  "nooutputboxerror",
  "norelax",
  "normalizelinemode",
  "normalizeparmode",
  "noscript",
  "nospaces",
  "nosubprescript",
  "nosubscript",
  "nosuperprescript",
  "nosuperscript",
  "novrule",
  "numericscale",
  "numericscaled",
  "numexperimental",
  "numexpression",
  "optionalboundary",
  "orelse",
  "orphanlinefactors",
  "orphanpenalties",
  "orunless",
  "outputbox",
  "overloaded",
  "overloadmode",
  "overshoot",
  "pageboundary",
  "pageexcess",
  "pageextragoal",
  "pagefistretch",
  "pagelastdepth",
  "pagelastfilllstretch",
  "pagelastfillstretch",
  "pagelastfilstretch",
  "pagelastfistretch",
  "pagelastheight",
  "pagelastshrink",
  "pagelaststretch",
  "pagevsize",
  "parametercount",
  "parameterdef",
  "parameterindex",
  "parametermark",
  "parametermode",
  "parattribute",
  "pardirection",
  "parfillleftskip",
  "parfillrightskip",
  "parinitleftskip",
  "parinitrightskip",
  "paroptions",
  "parpasses",
  "parpassesexception",
  "parshapewidth",
  "permanent",
  "pettymuskip",
  "positdef",
  "postexhyphenchar",
  "posthyphenchar",
  "postinlinepenalty",
  "postshortinlinepenalty",
  "prebinoppenalty",
  "predisplaygapfactor",
  "preexhyphenchar",
  "prehyphenchar",
  "preinlinepenalty",
  "prerelpenalty",
  "preshortinlinepenalty",
  "previousloopiterator",
  "primescript",
  "protecteddetokenize",
  "protectedexpandeddetokenize",
  "protrudechars",
  "protrusionboundary",
  "pxdimen",
  "quitloop",
  "quitloopnow",
  "quitvmode",
  "rdivide",
  "rdivideby",
  "realign",
  "resetlocalboxes",
  "resetmathspacing",
  "restorecatcodetable",
  "retained",
  "retokenized",
  "rightmarginkern",
  "righttwindemerits",
  "rpcode",
  "savecatcodetable",
  "scaledemwidth",
  "scaledexheight",
  "scaledextraspace",
  "scaledfontcharba",
  "scaledfontchardp",
  "scaledfontcharht",
  "scaledfontcharic",
  "scaledfontcharta",
  "scaledfontcharwd",
  "scaledfontdimen",
  "scaledinterwordshrink",
  "scaledinterwordspace",
  "scaledinterwordstretch",
  "scaledmathaxis",
  "scaledmathemwidth",
  "scaledmathexheight",
  "scaledmathstyle",
  "scaledslantperpoint",
  "scantextokens",
  "scriptspaceafterfactor",
  "scriptspacebeforefactor",
  "scriptspacebetweenfactor",
  "semiexpand",
  "semiexpanded",
  "semiprotected",
  "setdefaultmathcodes",
  "setfontid",
  "setmathatomrule",
  "setmathdisplaypostpenalty",
  "setmathdisplayprepenalty",
  "setmathignore",
  "setmathoptions",
  "setmathpostpenalty",
  "setmathprepenalty",
  "setmathspacing",
  "shapingpenaltiesmode",
  "shapingpenalty",
  "shortinlinemaththreshold",
  "shortinlineorphanpenalty",
  "showcodestack",
  "showstack",
  "singlelinepenalty",
  "snapshotpar",
  "spacechar",
  "spacefactormode",
  "spacefactoroverload",
  "spacefactorshrinklimit",
  "spacefactorstretchlimit",
  "specificationdef",
  "splitextraheight",
  "splitlastdepth",
  "splitlastheight",
  "splitlastshrink",
  "splitlaststretch",
  "srule",
  "subprescript",
  "subscript",
  "superprescript",
  "superscript",
  "supmarkmode",
  "swapcsvalues",
  "tabsize",
  "textdirection",
  "thewithoutunit",
  "tinymuskip",
  "tocharacter",
  "toddlerpenalties",
  "todimension",
  "tohexadecimal",
  "tointeger",
  "tokenized",
  "toksapp",
  "tokspre",
  "tolerant",
  "tolimitedfloat",
  "tomathstyle",
  "toscaled",
  "tosparsedimension",
  "tosparsescaled",
  "tpack",
  "tracingadjusts",
  "tracingalignments",
  "tracingbalancing",
  "tracingexpressions",
  "tracingfitness",
  "tracingfullboxes",
  "tracinghyphenation",
  "tracinginserts",
  "tracinglevels",
  "tracinglists",
  "tracinglooseness",
  "tracingmarks",
  "tracingmath",
  "tracingmvl",
  "tracingnodes",
  "tracingorphans",
  "tracingpasses",
  "tracingpenalties",
  "tracingtoddlers",
  "tsplit",
  "uleaders",
  "undent",
  "unexpandedendless",
  "unexpandedloop",
  "unexpandedrepeat",
  "unhpack",
  "unletfrozen",
  "unletprotected",
  "untraced",
  "unvpack",
  "variablefam",
  "vbadnessmode",
  "vbalance",
  "vbalancedbox",
  "vbalanceddeinsert",
  "vbalanceddiscard",
  "vbalancedinsert",
  "vbalancedreinsert",
  "vbalancedtop",
  "virtualhrule",
  "virtualvrule",
  "vpack",
  "vpenalty",
  "vsplitchecks",
  "wordboundary",
  "wrapuppar",
  "xdefcsname",
  "xtoks",
  "xtoksapp",
  "xtokspre",
 },
 ["omega"]={
  "Omegaminorversion",
  "Omegarevision",
  "Omegaversion",
 },
 ["pdftex"]={
  "ifpdfabsdim",
  "ifpdfabsnum",
  "ifpdfprimitive",
  "pdfadjustspacing",
  "pdfannot",
  "pdfcatalog",
  "pdfcolorstack",
  "pdfcolorstackinit",
  "pdfcompresslevel",
  "pdfcopyfont",
  "pdfcreationdate",
  "pdfdecimaldigits",
  "pdfdest",
  "pdfdestmargin",
  "pdfdraftmode",
  "pdfeachlinedepth",
  "pdfeachlineheight",
  "pdfendlink",
  "pdfendthread",
  "pdffirstlineheight",
  "pdffontattr",
  "pdffontexpand",
  "pdffontname",
  "pdffontobjnum",
  "pdffontsize",
  "pdfgamma",
  "pdfgentounicode",
  "pdfglyphtounicode",
  "pdfhorigin",
  "pdfignoreddimen",
  "pdfignoreunknownimages",
  "pdfimageaddfilename",
  "pdfimageapplygamma",
  "pdfimagegamma",
  "pdfimagehicolor",
  "pdfimageresolution",
  "pdfincludechars",
  "pdfinclusioncopyfonts",
  "pdfinclusionerrorlevel",
  "pdfinfo",
  "pdfinfoomitdate",
  "pdfinsertht",
  "pdflastannot",
  "pdflastlinedepth",
  "pdflastlink",
  "pdflastobj",
  "pdflastxform",
  "pdflastximage",
  "pdflastximagepages",
  "pdflastxpos",
  "pdflastypos",
  "pdflinkmargin",
  "pdfliteral",
  "pdfmajorversion",
  "pdfmapfile",
  "pdfmapline",
  "pdfminorversion",
  "pdfnames",
  "pdfnoligatures",
  "pdfnormaldeviate",
  "pdfobj",
  "pdfobjcompresslevel",
  "pdfomitcharset",
  "pdfomitcidset",
  "pdfomitinfodict",
  "pdfoutline",
  "pdfoutput",
  "pdfpageattr",
  "pdfpagebox",
  "pdfpageheight",
  "pdfpageref",
  "pdfpageresources",
  "pdfpagesattr",
  "pdfpagewidth",
  "pdfpkfixeddpi",
  "pdfpkmode",
  "pdfpkresolution",
  "pdfprimitive",
  "pdfprotrudechars",
  "pdfpxdimen",
  "pdfrandomseed",
  "pdfrecompress",
  "pdfrefobj",
  "pdfrefxform",
  "pdfrefximage",
  "pdfreplacefont",
  "pdfrestore",
  "pdfretval",
  "pdfsave",
  "pdfsavepos",
  "pdfsetmatrix",
  "pdfsetrandomseed",
  "pdfstartlink",
  "pdfstartthread",
  "pdfsuppressoptionalinfo",
  "pdfsuppressptexinfo",
  "pdftexbanner",
  "pdftexrevision",
  "pdftexversion",
  "pdfthread",
  "pdfthreadmargin",
  "pdftracingfonts",
  "pdftrailer",
  "pdftrailerid",
  "pdfuniformdeviate",
  "pdfuniqueresname",
  "pdfvorigin",
  "pdfxform",
  "pdfxformattr",
  "pdfxformmargin",
  "pdfxformname",
  "pdfxformresources",
  "pdfximage",
 },
 ["tex"]={
  " ",
  "-",
  "/",
  "above",
  "abovedisplayshortskip",
  "abovedisplayskip",
  "abovewithdelims",
  "accent",
  "adjdemerits",
  "advance",
  "afterassignment",
  "aftergroup",
  "atop",
  "atopwithdelims",
  "badness",
  "baselineskip",
  "batchmode",
  "begingroup",
  "belowdisplayshortskip",
  "belowdisplayskip",
  "binoppenalty",
  "botmark",
  "box",
  "boxmaxdepth",
  "brokenpenalty",
  "catcode",
  "char",
  "chardef",
  "cleaders",
  "clubpenalty",
  "copy",
  "count",
  "countdef",
  "cr",
  "crcr",
  "csname",
  "day",
  "deadcycles",
  "def",
  "defaulthyphenchar",
  "defaultskewchar",
  "delcode",
  "delimiter",
  "delimiterfactor",
  "delimitershortfall",
  "dimen",
  "dimendef",
  "discretionary",
  "displayindent",
  "displaylimits",
  "displaystyle",
  "displaywidowpenalty",
  "displaywidth",
  "divide",
  "doublehyphendemerits",
  "dp",
  "dump",
  "edef",
  "else",
  "emergencyextrastretch",
  "emergencystretch",
  "end",
  "endcsname",
  "endgroup",
  "endinput",
  "endlinechar",
  "eofinput",
  "eqno",
  "errhelp",
  "errmessage",
  "errorcontextlines",
  "errorstopmode",
  "escapechar",
  "everycr",
  "everydisplay",
  "everyhbox",
  "everyjob",
  "everymath",
  "everypar",
  "everyvbox",
  "exhyphenchar",
  "exhyphenpenalty",
  "expandafter",
  "fam",
  "fi",
  "finalhyphendemerits",
  "firstmark",
  "floatingpenalty",
  "font",
  "fontdimen",
  "fontname",
  "futurelet",
  "gdef",
  "global",
  "globaldefs",
  "halign",
  "hangafter",
  "hangindent",
  "hbadness",
  "hbox",
  "hfil",
  "hfill",
  "hfilneg",
  "hfuzz",
  "hkern",
  "holdinginserts",
  "hrule",
  "hsize",
  "hskip",
  "hss",
  "ht",
  "hyphenation",
  "hyphenchar",
  "hyphenpenalty",
  "if",
  "ifcase",
  "ifcat",
  "ifdim",
  "iffalse",
  "ifhbox",
  "ifhmode",
  "ifinner",
  "ifmmode",
  "ifnum",
  "ifodd",
  "iftrue",
  "ifvbox",
  "ifvmode",
  "ifvoid",
  "ifx",
  "ignorespaces",
  "indent",
  "input",
  "inputlineno",
  "insert",
  "insertpenalties",
  "interlinepenalty",
  "jobname",
  "kern",
  "language",
  "lastbox",
  "lastkern",
  "lastpenalty",
  "lastskip",
  "lccode",
  "leaders",
  "left",
  "lefthyphenmin",
  "leftskip",
  "leqno",
  "let",
  "limits",
  "linepenalty",
  "lineskip",
  "lineskiplimit",
  "long",
  "looseness",
  "lower",
  "lowercase",
  "mark",
  "mathaccent",
  "mathbin",
  "mathchar",
  "mathchardef",
  "mathchoice",
  "mathclose",
  "mathcode",
  "mathinner",
  "mathop",
  "mathopen",
  "mathord",
  "mathpunct",
  "mathrel",
  "mathsurround",
  "maxdeadcycles",
  "maxdepth",
  "meaning",
  "medmuskip",
  "message",
  "middle",
  "mkern",
  "month",
  "moveleft",
  "moveright",
  "mskip",
  "multiply",
  "muskip",
  "muskipdef",
  "newlinechar",
  "noalign",
  "noexpand",
  "noindent",
  "nolimits",
  "nonscript",
  "nonstopmode",
  "nulldelimiterspace",
  "nullfont",
  "number",
  "omit",
  "or",
  "outer",
  "output",
  "outputpenalty",
  "over",
  "overfullrule",
  "overline",
  "overwithdelims",
  "pagedepth",
  "pagefilllstretch",
  "pagefillstretch",
  "pagefilstretch",
  "pagegoal",
  "pageshrink",
  "pagestretch",
  "pagetotal",
  "par",
  "parfillskip",
  "parindent",
  "parshape",
  "parskip",
  "patterns",
  "pausing",
  "penalty",
  "postdisplaypenalty",
  "predisplaypenalty",
  "predisplaysize",
  "pretolerance",
  "prevdepth",
  "prevgraf",
  "radical",
  "raise",
  "relax",
  "relpenalty",
  "right",
  "righthyphenmin",
  "rightskip",
  "romannumeral",
  "scriptfont",
  "scriptscriptfont",
  "scriptscriptstyle",
  "scriptspace",
  "scriptstyle",
  "scrollmode",
  "setbox",
  "setlanguage",
  "sfcode",
  "shipout",
  "show",
  "showbox",
  "showboxbreadth",
  "showboxdepth",
  "showlists",
  "shownodedetails",
  "showthe",
  "skewchar",
  "skip",
  "skipdef",
  "spacefactor",
  "spaceskip",
  "span",
  "splitbotmark",
  "splitfirstmark",
  "splitmaxdepth",
  "splittopskip",
  "string",
  "tabskip",
  "textfont",
  "textstyle",
  "the",
  "thickmuskip",
  "thinmuskip",
  "time",
  "toks",
  "toksdef",
  "tolerance",
  "topmark",
  "topskip",
  "tracingcommands",
  "tracingloners",
  "tracinglostchars",
  "tracingmacros",
  "tracingonline",
  "tracingoutput",
  "tracingpages",
  "tracingparagraphs",
  "tracingrestores",
  "tracingstats",
  "uccode",
  "uchyph",
  "unboundary",
  "underline",
  "unhbox",
  "unhcopy",
  "unkern",
  "unpenalty",
  "unskip",
  "unvbox",
  "unvcopy",
  "uppercase",
  "vadjust",
  "valign",
  "vbadness",
  "vbox",
  "vcenter",
  "vfil",
  "vfill",
  "vfilneg",
  "vfuzz",
  "vkern",
  "vrule",
  "vsize",
  "vskip",
  "vsplit",
  "vss",
  "vtop",
  "wd",
  "widowpenalty",
  "xdef",
  "xleaders",
  "xspaceskip",
  "year",
 },
 ["xetex"]={
  "XeTeXversion",
 },
}