return {
 ["comment"]="% generated by mtxrun --script pattern --convert",
 ["exceptions"]={
  ["n"]=0,
 },
 ["metadata"]={
  ["mnemonic"]="hi",
  ["source"]="hyph-hi",
  ["texcomment"]="% no comment",
 },
 ["patterns"]={
  ["characters"]="ँंःअआइईउऊऋऌएऐओऔकखगघङचछजझञटठडढणतथदधनपफबभमयरलळवशषसहऽािीुूृॄेैोौ्॒॑ॠॡॢॣ‌‍",
  ["data"]="2‍2 1‌1 अ1 आ1 इ1 ई1 उ1 ऊ1 ऋ1 ॠ1 ऌ1 ॡ1 ए1 ऐ1 ओ1 औ1 ा1 ि1 ी1 ु1 ू1 ृ1 ॄ1 ॢ1 ॣ1 े1 ै1 ो1 ौ1 1क 1ख 1ग 1घ 1ङ 1च 1छ 1ज 1झ 1ञ 1ट 1ठ 1ड 1ढ 1ण 1त 1थ 1द 1ध 1न 1प 1फ 1ब 1भ 1म 1य 1र 1ल 1ळ 1व 1श 1ष 1स 1ह 2ँ1 2ं1 2ः1 2ऽ1 2॑1 2॒1 2्2",
  ["lefthyphenmin"]=1,
  ["length"]=358,
  ["n"]=70,
  ["righthyphenmin"]=1,
 },
 ["version"]="1.001",
}